/*
 * HikBuffer.cpp
 *
 *  Created on: 2014-6-4
 *      Author: weilinfeng
 */

#include "AudioBuffer.h"
#include "MCRPrint.h"
#include "HPR_Mutex.h"

CAudioBuffer::CAudioBuffer()
: m_pBufferBeginPtr(NULL)
, m_pBufferEndPtr(NULL)
, m_pReadPtr(NULL)
, m_pWritePtr(NULL)
, m_pInvalidPtr(NULL)
, m_bOneRound(false)
, m_bInit(false)
, m_iCount(0)
, m_iTotleSize(0)
{
}

CAudioBuffer::~CAudioBuffer()
{
    Release();
}

/*******************************************************************************
Function:       Initialize()
Description:    初始化缓冲
Input:
Output:
Return:         true-初始化成功  false-初始化失败
*******************************************************************************/
bool CAudioBuffer::Initialize()
{
    if (m_bInit)
    {
        return true;
    }
    
    m_pBufferBeginPtr = new unsigned char[AUDIO_BUFFER_SIZE];
    if (m_pBufferBeginPtr == NULL)
    {
        return false;
    }

    m_pBufferEndPtr    = m_pBufferBeginPtr + AUDIO_BUFFER_SIZE;
    m_pReadPtr         = m_pBufferBeginPtr;
    m_pWritePtr        = m_pBufferBeginPtr;
    //缓冲区无效空间的地址
    m_pInvalidPtr      = m_pBufferEndPtr;

    HPR_MutexCreate(&m_lock, HPR_MUTEX_FAST);

    m_bOneRound        = false;
    m_bInit            = true;

    m_iTotleSize       = 0;

    return true;
}

/*******************************************************************************
Function:       Release()
Description:    释放缓冲区内存
Input:          
Output:         
Return:         
*******************************************************************************/
void CAudioBuffer::Release()
{
    if (m_pBufferBeginPtr != NULL)
    {
        m_pBufferEndPtr = m_pBufferBeginPtr;
        m_pReadPtr      = m_pBufferBeginPtr;
        m_pWritePtr     = m_pBufferBeginPtr;
        delete[] m_pBufferBeginPtr;
        m_pBufferBeginPtr = NULL;
    }
    
    m_pInvalidPtr = NULL;
    
    m_bOneRound = false;
    m_bInit = false;
    
    m_iTotleSize = 0;

    HPR_MutexDestroy(&m_lock);
}

/** @fn WriteDataToBuffer
 *  @brief 从外部向缓冲区写数据
 *  @param pData - 待写入数据(IN)
 *  @param iWriteSize - 待写入数据长度(IN)
 *  @return 返回写入数据的长度
 */
int CAudioBuffer::WriteDataToBuffer(const unsigned char* pData, int iWriteSize)
{
    if (!m_bInit)
    {
        LOGD("CAudioBuffer->WriteDataToBuffer() no m_bInit");
        return 0;
    }
    
    if (pData == NULL || iWriteSize <= 0)
    {
        LOGD("CAudioBuffer->WriteDataToBuffer() pData == NULL");
        return 0;
    }
    
    HPR_MutexLock(&m_lock);
    int aLen = WriteData(pData, iWriteSize);
    HPR_MutexUnlock(&m_lock);
    return aLen;
}

/** @fn ReadDataFromBuffer
 *  @brief 从循环缓冲中读取数据
 *  @param iReadSize - 读取的数据长度(IN)
 *  @param pData - 读取的数据缓冲(OUT)
 *  @return 返回读取的数据长度
 */
int CAudioBuffer::ReadDataFromBuffer(int iReadSize, unsigned char* pData)
{
    if (!m_bInit)
    {
        return 0;
    }

    if (pData == NULL || iReadSize <= 0)
    {
    	LOGD("CAudioBuffer->WriteDataToBuffer() pData == NULL || iReadSize:%d", iReadSize);
        return 0;
    }

    HPR_MutexLock(&m_lock);
    int aLen = ReadData(iReadSize, pData);
    HPR_MutexUnlock(&m_lock);
    return aLen;
}

/*******************************************************************************
Function:       ReSet()
Description:    重置缓冲区
Input:
Output:
Return:         无
*******************************************************************************/
void CAudioBuffer::ReSet()
{
    if (m_pBufferBeginPtr != NULL)
    {
        memset(m_pBufferBeginPtr, 0, AUDIO_BUFFER_SIZE);
    }

    m_pBufferEndPtr = m_pBufferBeginPtr + AUDIO_BUFFER_SIZE;
    m_pReadPtr      = m_pBufferBeginPtr;
    m_pWritePtr     = m_pBufferBeginPtr;
    m_pInvalidPtr   = m_pBufferEndPtr;
    
    m_bOneRound     = false;

    m_iTotleSize    = 0;
}


/*******************************************************************************
Function:       WriteData()
Description:    将数据拷贝到缓冲中
Input:          pData       - 数据
                iDataLen    - 数据长度
Output:
Return:         写入到缓冲的长度
*******************************************************************************/
int CAudioBuffer::WriteData(const unsigned char* pData, int iDataLen)
{
	// 当开始写缓冲区的地址为空时，返回0
    if (m_pWritePtr == NULL)
    {
        LOGD("CAudioBuffer->WriteData m_pWritePtr == NULL");
        return 0;
    }
    
    //1.如果写和读之间没有差了一轮&&有足够的位置写入数据，则将带写入的数据全部写入
    //2.如果写和读之间没有差了一轮&&没有足够的位置写入数据，则直接将带写入的数据全部写入
    if (!m_bOneRound)
    {
        if (m_pWritePtr + iDataLen > m_pBufferEndPtr)
        {
            if (m_pBufferBeginPtr + iDataLen > m_pReadPtr)
            {
                LOGD("CAudioBuffer->WriteData  第一轮   写入数据大于从开始位置到写入位置的空间");
                return 0;
            }
            else
            {
                memcpy(m_pBufferBeginPtr, pData, iDataLen);
                
                m_pInvalidPtr = m_pWritePtr;
                
                m_pWritePtr = m_pBufferBeginPtr + iDataLen;
                
                m_bOneRound = true;

                LOGD("CAudioBuffer->WriteData 第一轮      写入数据大于从开始位置到写入位置的空间,写数据成功，变为第二轮");
            }
        } 
        else
        {
            memcpy(m_pWritePtr, pData, iDataLen);
            m_pWritePtr += iDataLen;
        }
    }
    else
    {
        if (m_pWritePtr + iDataLen > m_pReadPtr)
        {
            LOGD("CAudioBuffer->WriteData 第二轮      没有足够的内存空间去写数据:%d", m_iTotleSize);
            return 0;
        }
        else
        {
            memcpy(m_pWritePtr, pData, iDataLen);
            m_pWritePtr += iDataLen;
            LOGD("CAudioBuffer->WriteData 第二轮     m_pWritePtr + iDataLen <= m_pReadPtr");
        }
    }

    m_iTotleSize += iDataLen;
    return iDataLen;
}

/** @fn ReadData
 *  @brief 从缓冲区中读出数据
 *  @param iReadSize - 需要读出的数据长度(IN)
 *  @param pData - 读出数据的缓存(OUT)
 *  @return 读取的数据长度
 */
int CAudioBuffer::ReadData(int iReadSize, unsigned char* pData)
{
    if (iReadSize <= 0 || pData == NULL)
    {
    	LOGD("CAudioBuffer->ReadData pData缓冲为空，或者iReadSize读取的数据长度小于0");
        return 0;
    }

    if (m_pReadPtr == NULL || iReadSize > m_iTotleSize)
    {
    	LOGD("CAudioBuffer->ReadData 读取指针为空，或者读取的数据长度大于缓冲区的数据, readSize:%d, totleSize:%d", iReadSize, m_iTotleSize);
        return 0;
    }
    
    int aDataLen = 0;

    if (!m_bOneRound)
    {
        if (m_pReadPtr + iReadSize > m_pWritePtr)
        {
        	LOGD("CAudioBuffer->ReadData 第一轮     读取收据大于写入的数据了");
            return 0;
        }
        else
        {
            memcpy(pData, m_pReadPtr, iReadSize);
            m_pReadPtr += iReadSize;
            m_iTotleSize -= iReadSize;
            aDataLen = iReadSize;
            LOGD("CAudioBuffer->ReadData 第一轮     读取到数据");
        }
    }
    else
    {
        if (m_pReadPtr + iReadSize > m_pInvalidPtr)
        {
        	// 2014-10-14，wlf 解决此处存在问题，出现数据丢失的问题(丢失了较短的数据)
        	if((iReadSize - (m_pInvalidPtr-m_pReadPtr)) < (m_pWritePtr - m_pBufferBeginPtr)) {

        		memcpy(pData, m_pReadPtr, (m_pInvalidPtr - m_pReadPtr));

				memcpy(pData + (m_pInvalidPtr - m_pReadPtr), m_pBufferBeginPtr, iReadSize - (m_pInvalidPtr-m_pReadPtr));

				m_pReadPtr = m_pBufferBeginPtr + (iReadSize - (m_pInvalidPtr-m_pReadPtr));

				m_pInvalidPtr = m_pBufferEndPtr;

				m_iTotleSize = m_pWritePtr - m_pReadPtr;

				aDataLen = iReadSize;

				m_bOneRound = false;

				LOGD("CAudioBuffer->ReadData 第二轮       读取的数据大于前一轮剩余的数据, 进入第一轮");
        	}
        	else {
        		LOGD("CAudioBuffer->ReadData 第二轮       读取的数据大于剩余的数据");
        		return  0;
        	}
        }
        else
        {
            memcpy(pData, m_pReadPtr, iReadSize);
            m_pReadPtr += iReadSize;
            m_iTotleSize -= iReadSize;
            aDataLen = iReadSize;
            LOGD("CAudioBuffer->ReadData 第二轮      从前一轮读取数据成功");
        }
    }

    return aDataLen;
}
