/*
 * PushManager.cpp
 *
 *  Created on: 2014-3-4
 *      Author: weilinfeng
 */

#include "HPR_Select.h"
#include "HPR_Utils.h"
#include "MCRPrint.h"
#include "PushManager.h"
#include "HPR_Error.h"
#include "HPR_Netdb.h"

// IPV6 地址缓冲大小定义
#define IPV6_BUF_LEN    128

CPushManager::CPushManager()
: m_bStop(true)
, m_bQuit(false)
, m_sSession(NULL)
, m_pRecvBuf(NULL)
, m_pSendBuf(NULL)
, m_pBuffer(NULL)
, m_nWritePos(0)
, m_hRecvThreadHandle(NULL)
, m_hSendThreadHandle(NULL)
, m_pConnectionLister(NULL)
, m_pPacketLister(NULL)
, m_iSocket(HPR_INVALID_SOCKET)
, m_ip(0)
, m_port(0)
{
    m_Timeout.tv_sec = RECV_TIMEOUT;
    m_Timeout.tv_usec = 0;

    Init();
}

CPushManager::~CPushManager()
{
	Deinit();
}

/** @fn void Init()
 *   @brief 初始化
 *   @param - 无
 *   @return 无
 */
void CPushManager::Init()
{
	m_pRecvBuf = new char[PUSH_BUF_SIZE];
	m_pSendBuf = new char[PUSH_BUF_SIZE];
	m_sSession = new char[SESSION_SIZE];
	m_pBuffer = new char[PUSH_BUF_SIZE*10];
}

/** @fn void Deinit()
 *   @brief 初始化
 *   @param - 无
 *   @return 无
 */
void CPushManager::Deinit()
{
	if(m_pRecvBuf != NULL) {
		delete m_pRecvBuf;
		m_pRecvBuf = NULL;
	}

	if(m_pSendBuf != NULL) {
		delete m_pSendBuf;
		m_pSendBuf = NULL;
	}

	if(m_sSession != NULL) {
		delete m_sSession;
		m_sSession = NULL;
	}

	if(m_pBuffer != NULL) {
		delete m_pBuffer;
		m_pBuffer = NULL;
	}
}

/** @fn void SetPacketLister()
 *   @brief 设置数据包回调函数
 *   @param [in] - packetLister 数据包回调函数
 *   @return 无
 */
void CPushManager::SetPacketLister(pPacketLister packetLister) {
	CMCRPrint::Print("SetPacketLister\n");
	if(packetLister == NULL) {
		CMCRPrint::Print("SetPacketLister packetLister == NULL\n");
	}
	m_pPacketLister = packetLister;
}

/** @fn void SetPacketLister()
 *   @brief 设置连接回调函数
 *   @param [in] - connectionLister 连接回调函数
 *   @return 无
 */
void CPushManager::SetConnectionLister(pConnectionLister connectionLister) {
	CMCRPrint::Print("SetConnectionLister\n");
	if(connectionLister == NULL) {
		CMCRPrint::Print("SetConnectionLister connectionLister == NULL\n");
	}
	m_pConnectionLister = connectionLister;
}

/** @fn void Connect()
 *   @brief 建立长连接
 *   @param [in] - IP 长连接IP地址
 *   @param [in] - port 长连接端口
 *   @param [in] - sessionId msp回话
 *   @param [in] - pUser 用户数据
 *   @return < 0:连接失败，> 0：连接成功，返回Socket
 */
int CPushManager::Connect(const char *IP, int port, const char* sessionId, void* pUser) {
	CMCRPrint::Print("CPushManager->Connect\n");

	if(!m_bStop)
	{
		CMCRPrint::Print("CPushManager->Connect have Start \n");
		return m_iSocket;
	}

	if(IP == NULL || port == 0 || sessionId == NULL) {
		CMCRPrint::Print("CPushManager->Connect IP == NULL || port == 0 || sessionId == NULL\n");
		return -1;
	}

	if (m_sSession == NULL)
	{
		return -1;
	}

	CMCRPrint::Print(IP);
	CMCRPrint::Print("CPushManager->Connect port:%d\n", port);
	CMCRPrint::Print(sessionId);

	m_pUser = pUser;
	m_ip = inet_addr(IP);
	m_port = port;

	memset(m_sSession, 0, SESSION_SIZE);
	memcpy(m_sSession, sessionId, strlen(sessionId));

	// 连接服务器
    bool ret = false;
#ifdef HK_APPLE
    ret = ConnectServer(IP, port);
#else
    ret = ConnectServer();
#endif
	if(!ret)
	{
		CMCRPrint::Print("CPushManager->Connect ConnectServer fail \n");
		return -1;
	}

	// 开启收发线程
	ret = StartPushTrans();
	if(!ret)
	{
		CMCRPrint::Print("CPushManager->Connect StartPushTrans fail \n");
		DisConnectServer();
		return -1;
	}

	// 标识心跳线程没有停止
	m_bStop = false;

	CMCRPrint::Print("CPushManager->Connect success socket:%d \n", m_iSocket);
	return m_iSocket;
}

/** @fn void DisConnect()
 *   @brief 断开长连接
 *   @param 无
 *   @return 无
 */
void CPushManager::DisConnect()
{
	// 停止收发线程
	StopPushTrans();

	// 断开服务器
	DisConnectServer();

	m_bStop = true;
}

/** @fn void ConnectServer()
 *   @brief 连接服务器
 *   @param 无
 *   @return 无
 */
bool CPushManager::ConnectServer()
{
	m_iSocket = HPR_CreateSocket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(m_iSocket == HPR_INVALID_SOCKET) {
		CMCRPrint::Print("CPushManager->Connect create socket fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		return false;
	}

	// 设置套接字接收缓冲大小和发送缓冲大小
	HPR_INT32 ret = HPR_SetBuffSize(m_iSocket, 2*PUSH_BUF_SIZE, 2*PUSH_BUF_SIZE);
	if (ret != HPR_OK)
	{
		CMCRPrint::Print("CPushManager->Connect HPR_SetBuffSize fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		HPR_CloseSocket(m_iSocket);
		m_iSocket = HPR_INVALID_SOCKET;
		return false;
	}

	// 设置地址重用
	ret = HPR_SetReuseAddr(m_iSocket, true);
	if (ret != HPR_OK)
	{
		CMCRPrint::Print("CPushManager->Connect HPR_SetReuseAddr fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		HPR_CloseSocket(m_iSocket);
		m_iSocket = HPR_INVALID_SOCKET;
		return false;
	}

	// 通过地址族类型,字符串格式的地址及端口号组建HPR网络地址结构
	HPR_ADDR_T remoteaddr;
	memset(&remoteaddr, 0, sizeof(remoteaddr));
	ret = HPR_MakeAddr4ByInt(m_ip, m_port, &remoteaddr);
	if (ret != HPR_OK)
	{
		CMCRPrint::Print("CPushManager->Connect HPR_MakeAddrByString fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		HPR_CloseSocket(m_iSocket);
		m_iSocket = HPR_INVALID_SOCKET;
		return false;
	}

	// 连接服务器，超时时间是毫秒
	ret = HPR_ConnectWithTimeOut(m_iSocket, &remoteaddr, CONNECT_TIMEOUT);
	if (ret != HPR_OK)
	{
		CMCRPrint::Print("CPushManager->Connect connect fail, last error code:%d and system error code:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		HPR_CloseSocket(m_iSocket);
		m_iSocket = HPR_INVALID_SOCKET;
		return false;
	}

	CMCRPrint::Print("CPushManager->Connect success\n");

	return true;
}

/** @fn void ConnectServer()
 *   @brief 连接服务器
 *   @param 无
 *   @return 无
 */
bool CPushManager::ConnectServer(const char *ip, int port)
{
    HPR_ADDRESS_INFO *pAddressInfo = NULL;
    if ((HPR_GetAddressInfo(ip, NULL, NULL, &pAddressInfo) != 0))
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() HPR_GetAddressInfo is failed\n");
        return HPR_FALSE;
    }
    
    if (NULL == pAddressInfo)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() pAddressInfo is NULL\n");
        return HPR_FALSE;
    }
    
    HPR_ADDRESS_INFO *pAddrTmp = pAddressInfo;
    char szIPBUf[IPV6_BUF_LEN] = {0};
    while (pAddrTmp != NULL)
    {
        memset(szIPBUf, 0, IPV6_BUF_LEN);
        HPR_GetAddrStringEx(&(pAddrTmp->Address), szIPBUf, IPV6_BUF_LEN);
        
        // 开启ipv6解析域名，会出现回路地址：：1，过滤掉
        if (0 != strcmp(szIPBUf, "::1"))
        {
            break;
        }
        else
        {
            pAddrTmp = pAddrTmp->pNext;
        }
    }
    
    CMCRPrint::Print("CRtspClientEngine::ConnectServer() ip:[%s]\n", szIPBUf);
    
    bool bIPIsV6 = false;
    if (strstr(szIPBUf, ":") != NULL)
    {
        bIPIsV6 = true;
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() It is ipv6 address\n");
    }
    else
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() It is ipv4 address\n");
    }
    
    if (!bIPIsV6 && strlen(szIPBUf) > 16)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() ipV4 addr is invalide\n");
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    
    HPR_INT32 iAF = bIPIsV6 ? AF_INET6 : AF_INET;
    m_iSocket = HPR_CreateSocket(iAF, SOCK_STREAM, IPPROTO_TCP);
    if(m_iSocket == HPR_INVALID_SOCKET)
    {
        CMCRPrint::Print("CPushManager->Connect create socket fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
        return HPR_FALSE;
    }
    
    // 设置套接字接收缓冲大小和发送缓冲大小
    HPR_INT32 ret = HPR_SetBuffSize(m_iSocket, 2*PUSH_BUF_SIZE, 2*PUSH_BUF_SIZE);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CPushManager->Connect HPR_SetBuffSize fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
        HPR_CloseSocket(m_iSocket);
        m_iSocket = HPR_INVALID_SOCKET;
        return HPR_FALSE;
    }
    
    // 设置地址重用
    ret = HPR_SetReuseAddr(m_iSocket, true);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CPushManager->Connect HPR_SetReuseAddr fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
        HPR_CloseSocket(m_iSocket);
        m_iSocket = HPR_INVALID_SOCKET;
        return HPR_FALSE;
    }
    
    // 连接服务器，超时时间是毫秒
    pAddressInfo->Address.SA.sin4.sin_port = htons(port);
    pAddressInfo->Address.SA.sin6.sin6_port = htons(port);
    ret = HPR_ConnectWithTimeOut(m_iSocket, &pAddressInfo->Address, CONNECT_TIMEOUT);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CPushManager->Connect connect fail, last error code:%d and system error code:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
        HPR_CloseSocket(m_iSocket);
        m_iSocket = HPR_INVALID_SOCKET;
        return HPR_FALSE;
    }
    
    CMCRPrint::Print("CPushManager->Connect success\n");
    
    return true;
}
/** @fn void DisConnectServer()
 *   @brief 断开服务器
 *   @param 无
 *   @return 无
 */
void CPushManager::DisConnectServer()
{
	// 关闭socket
	if (m_iSocket != HPR_INVALID_SOCKET)
	{
		HPR_ShutDown(m_iSocket, HPR_SHUT_RDWR);
		HPR_CloseSocket(m_iSocket);
	}

	m_iSocket = HPR_INVALID_SOCKET;
}

/** @fn void StartPushTrans()
 *   @brief 开启Push数据交互
 *   @param 无
 *   @return 无
 */
bool CPushManager::StartPushTrans()
{
	CMCRPrint::Print("CPushManager->StartPushTrans\n");
	m_bQuit = false;

	m_hSendThreadHandle = HPR_Thread_Create(SendPushDataProc, this, 0);
	if (m_hSendThreadHandle == HPR_INVALID_HANDLE)
	{
		CMCRPrint::Print("CPushManager->StartPushTrans create push send thread fail\n");
		StopPushTrans();
		return false;
	}

	// 创建RTP发送线程
	m_hRecvThreadHandle = HPR_Thread_Create(RecvPushDataProc, this, 0);
	if (m_hRecvThreadHandle == HPR_INVALID_HANDLE)
	{
		CMCRPrint::Print("CPushManager->StartPushTrans create push recv thread fail\n");
		StopPushTrans();
		return false;
	}
	CMCRPrint::Print("CPushManager->StartPushTrans success\n");
	return true;
}

/** @fn void StopPushTrans()
 *   @brief 停止Push数据交互
 *   @param 无
 *   @return 无
 */
void CPushManager::StopPushTrans()
{
	CMCRPrint::Print("CPushManager->StopPushTrans\n");
    m_bQuit = true;

    // 停止RTP发送和接收线程
    if (m_hSendThreadHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
    {
        HPR_Thread_Wait(m_hSendThreadHandle);
        CMCRPrint::Print("CPushManager->StopPushTrans wait  end\n");
        m_hSendThreadHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
    }

    // 停止RTP发送和接收线程
    if (m_hRecvThreadHandle != (HPR_HANDLE)HPR_INVALID_THREAD)
    {
        HPR_Thread_Wait(m_hRecvThreadHandle);
        CMCRPrint::Print("CPushManager->StopPushTrans wait  end\n");
        m_hRecvThreadHandle = (HPR_HANDLE)HPR_INVALID_THREAD;
    }

    CMCRPrint::Print("CPushManager->StopPushTrans success\n");
}

/** @fn void SendPushDataProc()
 *   @brief 发送数据线程体
 *   @param [in] param
 *   @return 无
 */
HPR_VOIDPTR CALLBACK CPushManager::SendPushDataProc( HPR_VOIDPTR pParam )
{
	if(pParam == NULL) {
		return NULL;
	}

	CPushManager *pPushTrans = (CPushManager*)pParam;
	if (pPushTrans == NULL)
	{
		return NULL;
	}

	int iCount = 20;
	for (;;) {
		if (pPushTrans->m_bQuit) {
			CMCRPrint::Print("CPushManager::HeartBeatProc() Stop Heart Beat\n");
			break;
		}

		if (iCount >= 20) {
			iCount = 0;
			pPushTrans->SendHeartBeat();
		} else {
			HPR_Sleep(500);
			iCount++;
		}
	}
    
    return NULL;
}

/** @fn void SendHeartBeat()
 *   @brief 发送心跳
 *   @param [in] param
 *   @return 无
 */
bool CPushManager::SendHeartBeat() {
	CMCRPrint::Print("CPushManager->SendHeartBeat\n");

	if (m_pSendBuf == NULL)
	{
		CMCRPrint::Print("CPushManager->SendHeartBeat SendBuf is NULL\n");
		return false;
	}
	int len = GenerateHeartBeatPacket(m_pSendBuf);
	if (len == 0)
	{
		return false;
	}

	bool bRet = SendPacket(m_pSendBuf, len);
	if (!bRet)
	{
		return false;
	}

	return true;
}

/** @fn void GenerateHeartBeatPacket()
 *   @brief 生成心跳包
 *   @param [in] param
 *   @return 无
 */
int CPushManager::GenerateHeartBeatPacket(char* buf)
{
	if (buf == NULL || m_sSession == NULL)
	{
		return 0;
	}

	HEADER_INFO hdr;
	hdr.magic = 0xACE;
	hdr.channel = 0x00;
	hdr.length = sizeof(HB_REQ_INFO);
	int headLen = sizeof(HEADER_INFO);

	HB_REQ_INFO hbReq;
	memset(&hbReq, 0, sizeof(HB_REQ_INFO));
	memcpy(hbReq.mcuSessionId, m_sSession, strlen(m_sSession));
	hbReq.seesionIdLen = strlen(m_sSession);

	CMCRPrint::Print("CPushManager->HeartBeatPacket m_sSession is: %s\n", m_sSession);

	memset(buf, 0, PUSH_BUF_SIZE);
	memcpy(buf, (char*)(&hdr), headLen);
	memcpy(buf + headLen, (char*)(&hbReq), sizeof(HB_REQ_INFO));

	return headLen + hdr.length;
}

/** @fn void SendPacket()
 *   @brief 发送心跳包
 *   @param [in] 无
 *   @return 无
 */
bool CPushManager::SendPacket(char* buf , int len)
{
	//CMCRPrint::Print("CPushManager->SendPacket\n");

	if(buf == NULL || len == 0) {
		CMCRPrint::Print("CPushManager->SendPacket buf == NULL || len == 0\n");
		return false;
	}

	fd_set m_fdwrite;
	FD_ZERO(&m_fdwrite);
	FD_SET(m_iSocket, &m_fdwrite);

	if (HPR_Select(m_iSocket + 1, NULL, &m_fdwrite, NULL, &m_Timeout) <= 0)
	{
		CMCRPrint::Print("CPushManager->SendPacket HPR_Select()<=0, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		return false;
	}

	if (HPR_Send(m_iSocket, buf, HPR_INT32(len)) != HPR_INT32(len))
	{
		CMCRPrint::Print("CPushManager->SendPacket HPR_Send() fail, errorCode:%d and system errorCode:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		return false;
	}

	//CMCRPrint::Print("CPushManager->SendPacket HPR_Send() success\n");
	return true;
}

/** @fn void RecvPushDataProc()
 *   @brief 接受数据线程体
 *   @param [in] param
 *   @return 无
 */
HPR_VOIDPTR CALLBACK CPushManager::RecvPushDataProc(HPR_VOIDPTR pParam)
{
	CPushManager *pPushTrans = (CPushManager*)pParam;
    if (!pPushTrans)
    {
        CMCRPrint::Print("CPushManager->RecvPushDataProc pPushTrans == NULL\n");
        return NULL;
    }

    for (;;)
    {
        if (pPushTrans->m_bQuit)
        {
            CMCRPrint::Print("CPushManager->RecvPushDataProc Exit\n");
            break;
        }

        // 接收RTP报文
        pPushTrans->RecvPushData();
    }

    return NULL;
}

/** @fn void RecvPushData()
 *   @brief 接收函数
 *   @param [in] param
 *   @return 无
 */
bool CPushManager::RecvPushData()
{
	//	CMCRPrint::Print("CPushManager->RecvPushData\n");
	// 测试推送数据解析
	// test_ProcPushData();

	if (m_iSocket == HPR_INVALID_SOCKET)
	{
		CMCRPrint::Print("m_iSocket is HPR_INVALID_SOCKET\n");
		return false;
	}

	if (m_pRecvBuf == NULL)
	{
		CMCRPrint::Print("m_pRecvBuf is null\n");
		return false;
	}

    FD_ZERO(&m_fdread);
    FD_SET(m_iSocket, &m_fdread);
    memset(m_pRecvBuf, 0, PUSH_BUF_SIZE);
    HPR_UINT32 ret = HPR_Select(m_iSocket+1, &m_fdread, NULL, NULL, &m_Timeout);
    if (ret > 0)
    {
        HPR_INT32 readLen = HPR_Recv(m_iSocket, m_pRecvBuf, PUSH_BUF_SIZE);
        if (readLen > 0)
        {
//        	CMCRPrint::Print("CPushManager->RecvPushData readLen:%d\n", readLen);
        	ProcPushData(m_pRecvBuf, readLen);
        }// 返回0代表网络中断，Unix下，进程会收到一个SIGPIPE信号，进程对该信号的默认处理是进程终止
        else
        {
        	m_bQuit = true;
        	CMCRPrint::Print("CPushManager->RecvPushData HPR_Recv fail\n");
        	if (m_pConnectionLister != NULL)
        	{
        		CMCRPrint::Print("CPushManager->RecvPushData connection exception 100 \n");
        		(*m_pConnectionLister)(CONNECTION_EXCEPTION, m_pUser);
        	}
        }
    }
    else if (ret == 0)
    {
    	// 超时
    	// CMCRPrint::Print("CPushManager->RecvPushData::HPR_Select timeOut\n");
    }
    else
    {
    	m_bQuit = true;
    	CMCRPrint::Print("CPushManager->RecvPushData HPR_Select fail\n");
    	if(m_pConnectionLister != NULL) {
			(*m_pConnectionLister)(CONNECTION_EXCEPTION, m_pUser);
		}
    	else {
			CMCRPrint::Print("CPushManager->RecvPushData m_pConnectionLister == NULL\n");
		}
    }

    return true;
}

/** @fn void ProcPushData()
 *   @brief 数据处理函数
 *   @param [in] buf 数据缓冲
 *   @param [in] readlen 数据长度
 *   @return 无
 */
void CPushManager::ProcPushData(char* buf, unsigned int readlen)
{
//	CMCRPrint::Print("CPushManager::ProcPushData");

	if(buf == NULL) {
		CMCRPrint::Print("CPushManager::ProcPushData-> buf == NULL");
		return ;
	}

	if(readlen <= 0) {
		CMCRPrint::Print("CPushManager::ProcPushData-> readlen <= 0");
		return ;
	}

	if (m_pBuffer == NULL) {
		CMCRPrint::Print("CPushManager::ProcPushData-> buffer is null");
		return ;
	}

//	CMCRPrint::Print("CPushManager::ProcPushData readlen:%d", readlen);

	if (readlen < (PUSH_BUF_SIZE*10 - m_nWritePos)) {
		memcpy(m_pBuffer + m_nWritePos, buf, readlen);

		m_nWritePos += readlen;

		HPR_UINT16 packlen = 0;
		// 是否继续循环继续
		HPR_BOOL keeploop = HPR_TRUE;
		// 设置指向消息缓冲的指针
		char *pTmpPtr = m_pBuffer;
		// 获取数据头长度
		int headLen = sizeof(HEADER_INFO);
//		CMCRPrint::Print("CPushManager::ProcPushData HEADER_INFO headLen:%d", headLen);

		//因此缓存内必须至少有8(push头的长度)字节才足以解析，所以限制调整为大于等于8(push头的长度)
		while (keeploop && (m_nWritePos >= headLen)) {
			// 判断起始位置是否是消息头，如果有头数据且数据足够则回调
			if (IsHeadWithPush(pTmpPtr)) {
				memcpy(&packlen, pTmpPtr + (headLen-2), 2);
//				CMCRPrint::Print("CPushManager::ProcPushData packet packlen:%d", packlen);
				// 不做网络字节序的转化packlen = htons(packlen);
				if ((packlen + 8) <= m_nWritePos) {

					// 回调数据到上层
					CallDataBack(pTmpPtr, packlen);

					// 写入位置需要减去整个内容的长度
					m_nWritePos = m_nWritePos - packlen - headLen;
					//CMCRPrint::Print("CPushManager::ProcPushData m_nWritePos = m_nWritePos - packlen - headLen after, m_nWritePos:%d", m_nWritePos);
					pTmpPtr = pTmpPtr + packlen + headLen;

					if (m_nWritePos < headLen) {
						//CMCRPrint::Print("CPushManager::ProcPushData-> write position < 8\n");
						memcpy(m_pBuffer, pTmpPtr, m_nWritePos);
					}
				}
				else {
					if (pTmpPtr != m_pBuffer) {
						//CMCRPrint::Print("CPushManager::ProcPushData-> skip,and  m_nWritePos:%d ,packetlen:%d \n", m_nWritePos, packlen);
						memcpy(m_pBuffer, pTmpPtr, m_nWritePos);
					}
					//在数据不足以回调时退出循环，等待下一批数据填入缓存后重新解析
					keeploop = HPR_FALSE;
				}
			}
			else {
				HPR_UINT32 headpos = 1;
				// 循环到最后16字节之前，因为可能最后是一个不完整的头，留在后面数据补齐后再判断
				for ( ; headpos < (m_nWritePos - 7); ++ headpos) {
					if (IsHeadWithPush(pTmpPtr + headpos)){
						//CMCRPrint::Print("CPushManager::ProcPushData->find push TPKT and headpos:%d\n", headpos);
						break;
					}
					//CMCRPrint::Print("CPushManager::ProcPushData-> not find push TPKT\n");
				}

				//无论找到与否这里均作一次拷贝，是由于存在以下两种情况
				//1、找到了push头
				if (headpos < (m_nWritePos - 7)) {
					m_nWritePos -= headpos;
					//CMCRPrint::Print("CPushManager::ProcPushData-> m_nWritePos -= headpos after, m_nWritePos:%d\n", m_nWritePos);
					pTmpPtr += headpos;
				}//2、没有找到push头
				else {
					//此处的写位置应置为8，即之前排查没有查到的8个字节应该保留
					m_nWritePos = 8;
					memcpy(m_pBuffer, pTmpPtr + headpos, m_nWritePos);
					//缓存数据中没有push头，则退出循环
					keeploop = HPR_FALSE;
				}
				HPR_Sleep(10);
			}
		}
	}
	else {
		CMCRPrint::Print("CPushManager::ProcPushData-> buffer overflow");
	}
}

void CPushManager::CallDataBack(char* buf, int packlen)
{
	if (buf == NULL || packlen <= 0)
	{
		CMCRPrint::Print("buf == NULL or packlen <= 0");
		return;
	}

	HEADER_INFO phead;

	// 获取数据头长度
	int headLen = sizeof(HEADER_INFO);
	memset(&phead, 0, headLen);
	memcpy(&phead, buf, headLen);
	//0x00  表示心跳；
	if (phead.channel == 0x00)
	{
//		CMCRPrint::Print("CPushManager::ProcPushData-> 心跳包");
	}//0x01 表示需要透传的数据
	else if (phead.channel == 0x01)
	{
		CMCRPrint::Print("CPushManager::ProcPushData-> 需要透传的数据包");
	}//0x02 表示需要反向消息通知数据
	else if (phead.channel == 0x02)
	{
		if (m_pPacketLister != NULL)
		{
//			CMCRPrint::Print("CPushManager::ProcPushData-> pPacketLister call back");
//			CMCRPrint::Print("CPushManager::ProcPushData-> packlen:%d, headLen:%d,\n", packlen, headLen);
			(*m_pPacketLister)(PUSH_PACKET_TYPE, buf + headLen, packlen-headLen, m_pUser);
		}
//		CMCRPrint::Print("CPushManager::ProcPushData->反向消息通知数据");
	}
}

bool CPushManager::IsHeadWithPush(char const *src) {
	HEADER_INFO phead;
	memset(&phead, 0, sizeof(HEADER_INFO));
	memcpy(&phead, src, sizeof(HEADER_INFO));
	if (phead.magic == 0xACE) {
		return HPR_TRUE;
	}
	else {
		return HPR_FALSE;
	}
}

 // 测试消息解析
void CPushManager::test_ProcPushData() {
	HEADER_INFO phead;
	memset(&phead, 0, sizeof(HEADER_INFO));
	phead.magic = 0xACE;
	phead.channel = 0x02;
	phead.length = sizeof(RREVERSE_NOTIC_REQ_INFO);

	RREVERSE_NOTIC_REQ_INFO pdata;
	memset(&pdata, 0, sizeof(RREVERSE_NOTIC_REQ_INFO));
	memcpy(pdata.mcuSessionId, &"123456789000", 12);
	pdata.seesionIdLen = 12;
	memcpy(pdata.messageId, &"10000", 5);
	pdata.messageIdLen = 5;
	memcpy(pdata.messageTitle, &"aabbccddeeffgg", 14);
	pdata.messageTitleLen = 14;
	memcpy(pdata.messageType, &"DM", 2);
	pdata.messageTypeLen = 2;
	memcpy(pdata.messageTypeDescribe, &"ddmmddmm", 8);
	pdata.messageTypeLen = 8;
	time_t ctime;
	time(&ctime);
	pdata.messageCreateTime = ctime;

	if (m_pRecvBuf == NULL)
	{
		CMCRPrint::Print("m_pRecvBuf is null\n");
		return;
	}
	// 获取数据头长度
	memcpy(m_pRecvBuf, &phead, sizeof(HEADER_INFO));
	memcpy(m_pRecvBuf + sizeof(HEADER_INFO), &pdata, sizeof(RREVERSE_NOTIC_REQ_INFO));

	ProcPushData(m_pRecvBuf, sizeof(RREVERSE_NOTIC_REQ_INFO)+sizeof(HEADER_INFO));
}



