/*
* rtp.h -- RTP header file
*/

#ifndef __RTP_RTCP_H__
#define __RTP_RTCP_H__

/*
* The type definitions below are valid for 32-bit architectures and
* may have to be adjusted for 16- or 64-bit architectures.
*/
typedef unsigned char	u_int8;
typedef unsigned short	u_int16;
typedef unsigned int	u_int32;
typedef short			int16;

#define lit_endian		0x0123
#define big_endian		0x3210
#define vod_byte_order	lit_endian

/*
* Current protocol version.
*/
#define RTP_VERSION		2

#define RTP_SEQ_MOD		(1 << 16)	
#define RTP_MAX_SDES	255			/* maximum text length for SDES */


/*
* RTP payload type
*/
typedef enum
{
	rtpPayloadPCMU = 0,
	rtpPayload1016 = 1,
	rtpPayloadG726_32 = 2,
	rtpPayloadGSM = 3,
	rtpPayloadG723 = 4,
	rtpPayloadDVI4_8KHz = 5,
	rtpPayloadDVI4_16KHz = 6,
	rtpPayloadLPC = 7,
	rtpPayloadPCMA = 8,
	rtpPayloadG722 = 9,
	rtpPayloadL16_stereo = 10,
	rtpPayloadL16_mono = 11,
	rtpPayloadQCELP = 12,
	rtpPayloadMPA = 14,
	rtpPayloadG728 = 15,
	rtpPayloadDVI4_11KHz = 16,
	rtpPayloadDVI4_22KHz = 17,
	rtpPayloadG729 = 18,
	rtpPayloadCN = 19,
	rtpPayloadGSMEFR = 20,
	rtpPayloadG726_40 = 21,
	rtpPayloadG726_24 = 22,
	rtpPayloadG726_16 = 23,
	rtpPayloadCelB = 25,
	rtpPayloadJPEG = 26,
	rtpPayloadNV = 28,
	rtpPayloadH261 = 31,
	rtpPayloadMPV = 32,
	rtpPayloadMP2T = 33,
	rtpPayloadH263 = 34,
	rtpPayloadDynMin = 96,
	rtpPayLoadDyn98 = 98,
	rtpPayLoadDyn99 = 99,
	rtpPayLoadDyn100 = 100,
	rtpPayloadDTMF_RFC2833 = 100,
	rtpPayLoadDyn101 = 101,
	rtpPayloadCiscoRtp = 121,
	rtpPayloadL16_8k_mono = 122,
	rtpPayloadDynMax = 127,         /* max int of 7-bit unsigned int */
	rtpPayloadUndefined = 128    
} RtpPayloadType;

typedef enum 
{
	RTCP_SR = 200,
	RTCP_RR = 201,
	RTCP_SDES = 202,
	RTCP_BYE = 203,
	RTCP_APP = 204
} rtcp_type;

typedef enum 
{
	RTCP_SDES_END = 0,
	RTCP_SDES_CNAME = 1,
	RTCP_SDES_NAME = 2,
	RTCP_SDES_EMAIL = 3,
	RTCP_SDES_PHONE = 4,
	RTCP_SDES_LOC = 5,
	RTCP_SDES_TOOL = 6,
	RTCP_SDES_NOTE = 7,
	RTCP_SDES_PRIV = 8
} rtcp_sdes_type;


/*
* RTP data header
*/
typedef struct 
{
#if vod_byte_order == big_endian
	u_int32 version		: 2;		/* protocol version */
	u_int32 padding		: 1;		/* padding flag */
	u_int32 extension	: 1;		/* header extension flag */
	u_int32 count		: 4;		/* CSRC count */
	u_int32 marker		: 1;		/* marker bit */
	u_int32 payload		: 7;		/* payload type */
	u_int32 sequence	: 16;		/* sequence number */
#elif vod_byte_order == lit_endian
	u_int32 count		: 4;		/* CSRC count */
	u_int32 extension	: 1;		/* header extension flag */
	u_int32 padding		: 1;		/* padding flag */
	u_int32 version		: 2;		/* protocol version */
	u_int32 payload		: 7;		/* payload type */
	u_int32 marker		: 1;		/* marker bit */
	u_int32 sequence	: 16;		/* sequence number */
#endif

	u_int32 timestamp;				/* timestamp */
	u_int32 ssrc;					/* synchronization source */
} rtp_hdr_t;

/*
* RTP extension header
*/
typedef struct 
{
	unsigned short id;
	unsigned short len;
} rtp_ext_hdr_t;


/*
* RTCP common header word
*/
typedef struct 
{
#if vod_byte_order == big_endian
	u_int8 version : 2;				/* protocol version */
	u_int8 padding : 1;				/* padding flag */
	u_int8 count   : 5;				/* varies by packet type */
#elif vod_byte_order == lit_endian
	u_int8 count   : 5;				/* varies by packet type */
	u_int8 padding : 1;				/* padding flag */
	u_int8 version : 2;				/* protocol version */
#endif
	u_int8 type;					/* RTCP packet type */
	u_int16 length;					/* pkt len in words, w/o this word */
} rtcp_common_t;

/*
* Big-endian mask for version, padding bit and packet type pair
*/
#define RTCP_VALID_MASK		(0xc000 | 0x2000 | 0xfe)
#define RTCP_VALID_VALUE	((RTP_VERSION << 14) | RTCP_SR)

/*
* Reception report block
*/
typedef struct 
{
	u_int32 ssrc;				/* data source being reported */
	u_int32 fraction:8;			/* fraction lost since last SR/RR */
	int lost:24;				/* cumul. no. pkts lost (signed!) */
	u_int32 last_seq;			/* extended last seq. no. received */
	u_int32 jitter;				/* interarrival jitter */
	u_int32 lsr;				/* last SR packet from this source */
	u_int32 dlsr;				/* delay since last SR packet */
} rtcp_rr_item_t;

/*
* SDES item
*/
typedef struct 
{
	u_int8 type;				/* type of item (rtcp_sdes_type_t) */
	u_int8 length;				/* length of item (in octets) */
	char data[2];				/* text, not null-terminated */
} rtcp_sdes_item_t;

/*
* Application Description
*/
typedef struct  
{
	u_int8 name[4];				/* app name */
	//u_int8 app_spe[4];		/* application-dependent data */
} rtcp_app_item_t;


typedef struct 
{
	u_int32 ssrc;		/* sender generating this report */
	u_int32 ntp_sec;	/* NTP timestamp */
	u_int32 ntp_frac;
	u_int32 rtp_ts;		/* RTP timestamp */
	u_int32 psent;		/* packets sent */
	u_int32 osent;		/* octets sent */
	rtcp_rr_item_t item[1];	/* variable-length list */
} rtcp_sr_t;

typedef struct 
{
	u_int32 ssrc;		/* receiver generating this report */
	rtcp_rr_item_t item[1];	/* variable-length list */
} rtcp_rr_t;

typedef struct 
{
	u_int32 src;		/* first SSRC/CSRC */
	rtcp_sdes_item_t item[1];	/* list of SDES items */
} rtcp_sdes_t;

typedef struct 
{
	u_int32 src;		/* first SSRC/CSRC */
	rtcp_app_item_t item[1];	/* list of APP items */
} rtcp_app_t;

typedef struct 
{
	u_int32 src[1];		/* list of sources */
	/* can't express trailing text for reason */
} rtcp_bye_t;

/*
* One RTCP packet
*/
typedef struct 
{
	rtcp_common_t common;		/* common header */

	union 
	{
		/* sender report (SR) */
		rtcp_sr_t sr;
		
		/* reception report (RR) */
		rtcp_rr_t rr;

		/* source description (SDES) */
		rtcp_sdes_t sdes;

		/* application-defined*/
		rtcp_app_t app;

		/* BYE */
		rtcp_bye_t bye;
	} r;
} rtcp_t;

/*
* Per-source state information
*/
typedef struct 
{
	u_int16 max_seq;			/* highest seq. number seen */
	u_int32 cycles;				/* shifted count of seq. number cycles */
	u_int32 base_seq;			/* base seq number */
	u_int32 bad_seq;			/* last 'bad' seq number + 1 */
	u_int32 probation;			/* sequ. packets till source is valid */
	u_int32 received;			/* packets received */
	u_int32 expected_prior;		/* packet expected at last interval */
	u_int32 received_prior;		/* packet received at last interval */
	u_int32 transit;			/* relative trans time for prev pkt */
	u_int32 jitter;				/* estimated jitter */
	/* ... */
} source;

#endif
