/**	@file 	RtspClientEngine.cpp
 *	@note 	HangZhou Hikvision System Technology Co., Ltd. All Right Reserved.
 *	@brief 	RtspClientEngine 负责信令交互
 *
 *	@author	weilinfeng
 *	@date	2013-05-20
 *
 *
 */
#include "HPR_Select.h"
#include "HPR_Netdb.h"
#include "RtspClientEngine.h"
#include "RtspDefine.h"
#include "MCRPrint.h"
#include "MsgManage.h"
#include "base64.h"
#include <math.h>

#include "HPR_Addr.h"
#include "HPR_Types.h"
#include "HPR_Netdb.h"
#include "HPR_Socket.h"

int g_nRtspClientErrorCode;

CRtspClientEngine::CRtspClientEngine()
: m_bInit(HPR_FALSE)
, m_bPlaying(HPR_FALSE)
, m_iEngineerIndex(-1)
, m_iRtpProtocol(RTPRTSP_TRANSMODE)
, m_iRTSPSocket(HPR_INVALID_SOCKET)
, m_bStopOptions(HPR_TRUE)
, m_bVTDUStackErr(HPR_FALSE)
, m_pDataCallback(NULL)
, m_pMsgCallback(NULL)
, m_iVersion(VERSION_VTDU_4)
, m_UserId(NULL)
, m_iSeq(0)
, m_iRTSPServerAddr(0)
, m_iRTSPServerPort(0)
, m_iRecvTotalLen(0)
, m_sSendBuf(NULL)
, m_sRecvBuf(NULL)
, m_sRtspUrl(NULL)
, m_sLocationUrl(NULL)
, m_sSession(NULL)
, m_Auth(NULL)
, m_sTransportBuf(NULL)
, m_sServerPortBuf(NULL)
, m_sInterleavedBuf(NULL)
, m_iSSRC(0)
, m_UserAgent(NULL)
, m_pRtpUdpTransEngine(NULL)
, m_pRtcpUdpTransEngine(NULL)
, m_pRtpTcpTransEngine(NULL)
, m_pRtpRtcpTransEngine(NULL)
, m_hOptionsThread(HPR_INVALID_HANDLE)
, m_state(RTSPSTATE_TEARDOWN)
, m_bretrans(false)
{
	memset(&m_oSdpInfo, 0, sizeof(m_oSdpInfo));
	memset(&m_sRTSPServerAddr, 0, IPADDR_LEN);
}

CRtspClientEngine::~CRtspClientEngine()
{
}


/** @fn bool Init()
 *   @brief 初始化RTSP引擎
 *   @param [in] iIndex - 引擎ID
 *   @param [in] iBasePort - 基础端口rtp
 *   @param [in] iRtpProtocol - rtp传输协议类型
 *   @param [in] pDataCallback - 数据回调
 *   @param [in] pMsgCallback - 消息回调
 *   @param [in] userId - 用户数据
 *   @param [out] 无
 *   @return true - 初始化成功; false - 初始化库失败
 */
HPR_BOOL CRtspClientEngine::Init(int iIndex, int iBasePort, int iRtpProtocol, pRtspDataCallback pDataCallback, pRtspMsgCallback pMsgCallback, void* userId)
{
	if (m_bInit)
	{
		LOGD("CRtspClientEngine::Init() have been inited\n");
		return HPR_TRUE;
	}

	if (iIndex < 0 || iIndex >= RTSPCLIENT_MAXNUM)
	{
		LOGD("CRtspClientEngine::Init() parameter error\n");
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEID_INVALID;
		return HPR_FALSE;
	}

	if(iBasePort < 0) {
		LOGD("CRtspClientEngine::Init() parameter error\n");
		g_nRtspClientErrorCode = RTSPCLIENT_BASEPORT_ERROR;
		return HPR_FALSE;
	}


	if (iRtpProtocol != RTPRTSP_TRANSMODE && iRtpProtocol != RTPUDP_TRANSMODE && iRtpProtocol != RTPTCP_TRANSMODE)
    {
        LOGD("CRtspClientEngine::Init() protocol not support\n");
        g_nRtspClientErrorCode = RTSPCLIENT_PROTOCOL_NOTSUPPORT;
        return HPR_FALSE;
    }

	m_iEngineerIndex = iIndex;
	m_iRtpProtocol = iRtpProtocol;
	m_pDataCallback = pDataCallback;
	m_pMsgCallback = pMsgCallback;
	m_UserId = userId;
	m_UserAgent = (char*)"PhoneClient";

	for(int i = 0; i < MEDIA_INFO_MAX_NUM; i++)
	{
		m_oUdpTcpPortPair[i].SetRTPClientPort(iBasePort + iIndex * MEDIA_INFO_MAX_NUM*2 + i);
		m_oUdpTcpPortPair[i].SetRTCPClientPort(iBasePort + iIndex * MEDIA_INFO_MAX_NUM*2 + i + 1);
	}

	m_oTCPInterleavedPair.SetRTPChannel(0x00);
	m_oTCPInterleavedPair.SetRTCPChannel(0x01);

	HPR_BOOL bRet = MallocMemory();
	if (!bRet)
	{
        g_nRtspClientErrorCode = RTSPCLIENT_MALLOC_MEMORY_FAIL;
        LOGD("CRtspClientEngine::Init() malloc fail\n");
		return HPR_FALSE;
	}

	m_bInit = HPR_TRUE;
	m_bPlaying = HPR_FALSE;

	return HPR_TRUE;
}

/** @fn bool Release()
 *   @brief 释放RTSP引擎
 *   @param [in] 无
 *   @param [out] 无
 *   @return 无
 */
HPR_BOOL CRtspClientEngine::Release()
{
	if (!m_bInit)
	{
		LOGD("CRtspClientEngine::Release() not init \n");
		return HPR_TRUE;
	}

	if(m_bPlaying)
	{
		LOGD("CRtspClientEngine::Release() be playing\n");
		StopRtspProc();
	}

	FreeMemory();

	m_pDataCallback = NULL;
	m_pMsgCallback	= NULL;

	m_bInit = HPR_FALSE;
	m_bPlaying = HPR_FALSE;

	LOGD("CRtspClientEngine::Release() success\n");

	return HPR_TRUE;
}

/** @fn bool StartRtspProc()
 *   @brief 开始rtsp交互
 *   @param [in] pRtspURL - rtsp预览地址
 *   @param [in] devicename - 设备名称
 *   @param [in] devicepsw - 设备密码
 *   @param [out] 无
 *   @return true -开始成功; false - 开始失败
 */
HPR_BOOL CRtspClientEngine::StartRtspProc(const char* pRtspURL, const char* devicename, const char* devicepsw)
{
	LOGD("CRtspClientEngine::StartRtspProc() RtspUrl:\n");
	LOGD(pRtspURL);

	if (!m_bInit)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEER_NOINIT;
		LOGD("CRtspClientEngine::SartRtspProc Not Init\n");
		return HPR_FALSE;
	}

	if (m_bPlaying)
	{
		LOGD("CRtspClientEngine::SartRtspProc Is Playing\n");
		return HPR_TRUE;
	}

	if(pRtspURL == NULL || strlen(pRtspURL) > MAX_URL_LEN) {
		LOGD("CRtspClientEngine::SartRtspProc url > 1024\n");
		return HPR_FALSE;
	}

	memset(m_sRtspUrl, 0, MAX_URL_LEN);
	memcpy(m_sRtspUrl, pRtspURL, strlen(pRtspURL));

	char deviceName[MAX_USERNAME_LEN] = {0};
	memset(deviceName, 0, MAX_USERNAME_LEN);
	if(devicename != NULL && strlen(devicename) < MAX_USERNAME_LEN)
	{
		memcpy(deviceName, devicename, strlen(devicename));
	}
	else
	{
		LOGD("start rtsp but dev name is NULL.");
	}

	char devicePassword[MAX_USERNAME_LEN] = {0};
	memset(devicePassword, 0, MAX_USERNAME_LEN);
	if(devicepsw != NULL && strlen(devicepsw) < MAX_USERNAME_LEN)
	{
		memcpy(devicePassword, devicepsw, strlen(devicepsw));
	}
	else
	{
		LOGD("start rtsp but dev pwd is NULL.");
	}

//重定向
DESCIRB:

	if (!RecognRealVersion(m_sRtspUrl))
	{
		LOGD("CRtspClientEngine::RecognRealVersion fail\n");
		return HPR_FALSE;
	}

	HPR_BOOL bRet = ParseURL(m_sRtspUrl, deviceName, devicePassword);
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_RTSPURL_ERROR;
		LOGD("CRtspClientEngine::SartRtspProc ParseURL Fail\n");
		return HPR_FALSE;
	}

	bRet = GenerateAuth(deviceName, devicePassword);
	if (!bRet) {
		LOGD("CRtspClientEngine::SartRtspProc GenerateAuth Fail\n");
	}

	if (ConnectServer() == HPR_ERROR)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_CONNECT_SERVER_FAIL;
		LOGD("CRtspClientEngine::SartRtspProc ConnectServer Fail\n");
		return HPR_FALSE;
	}

	bRet = Describe();
	if (!bRet)
	{
		if (m_statusCode == RTSP_STATUS_302 || m_statusCode == RTSP_STATUS_301)
		{
			bRet = GetRedirectUrl();
			if (bRet)
			{
				DisConnectServer();
				goto DESCIRB;
			}
		}
		DisConnectServer();
		LOGD("CRtspClientEngine::SartRtspProc Describe Fail\n");
		return HPR_FALSE;
	}

	bRet = Setup();
	if (!bRet)
	{
		DisConnectServer();
		LOGD("CRtspClientEngine::SartRtspProc Setup Fail\n");
		return HPR_FALSE;
	}

	bRet = Play();
	if (!bRet)
	{
		DisConnectServer();
		LOGD("CRtspClientEngine::SartRtspProc Play Fail\n");
		return HPR_FALSE;
	}

	if (m_iVersion == VERSION_VTDU_4)
	{
		bRet = ForceIFrame();
		if(!bRet)
		{
			LOGD("CRtspClientEngine::SartRtspProc ForceIFrame Fail\n");
		}
	}

	StartOptionsThread();

	m_bPlaying = HPR_TRUE;

	return HPR_TRUE;
}

/** @fn bool StopRtspProc()
 *   @brief 停止rtsp
 *   @param [in] 无
 *   @param [out] 无
 *   @return true -开始成功; false - 开始失败
 */
HPR_BOOL CRtspClientEngine::StopRtspProc()
{
	if (!m_bInit) {
		LOGD("CRtspClientEngine::StopRtspProc fail, no init\n");
		return HPR_FALSE;
	}

	StopOptionsThread();

	StopRtpTcpTrans();

	StopRtpUdpTrans();

	StopRtpRtspTrans();

	if (m_bPlaying)
	{
		TearDown();
	}

	DisConnectServer();

	m_bPlaying = HPR_FALSE;

	LOGD("CRtspClientEngine::StopRtspProc success \n");

	return HPR_TRUE;
}

// 返回rtsp回话ID
char* CRtspClientEngine::GetSession() {
	return m_sSession;
}

// 按时间回放
HPR_BOOL CRtspClientEngine::PlaybackByTime(const char* pRtspURL, const char* devicename, const char* devicepsw, pABS_TIME pfrom, pABS_TIME pto)
{
	LOGD("CRtspClientEngine::PlaybackByTime() RtspUrl:\n");
	LOGD(pRtspURL);
	LOGD("CRtspClientEngine::PlaybackByTime() devicename:%s\n", const_cast<char*>(devicename));
	LOGD("CRtspClientEngine::PlaybackByTime() devicepsw:%s\n", const_cast<char*>(devicepsw));

	if (!m_bInit)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_ENGINEER_NOINIT;
		LOGD("CRtspClientEngine::PlaybackByTime Not Init\n");
		return HPR_FALSE;
	}

	if (m_bPlaying)
	{
		LOGD("CRtspClientEngine::PlaybackByTime Is Playing\n");
		return HPR_TRUE;
	}

	if(pRtspURL == NULL || strlen(pRtspURL) > MAX_URL_LEN) {
		LOGD("CRtspClientEngine::SartRtspProc url > 1024\n");
		return HPR_FALSE;
	}

	memset(m_sRtspUrl, 0, MAX_URL_LEN);
	memcpy(m_sRtspUrl, pRtspURL, strlen(pRtspURL));

	char deviceName[MAX_USERNAME_LEN];
	memset(deviceName, 0, MAX_USERNAME_LEN);
	if(devicename != NULL && strlen(devicename) < MAX_USERNAME_LEN) {
		memcpy(deviceName, devicename, strlen(devicename));
	}

	char devicePassword[MAX_USERNAME_LEN];
	memset(devicePassword, 0, MAX_USERNAME_LEN);
	if(devicepsw != NULL && strlen(devicepsw) < MAX_USERNAME_LEN) {
		memcpy(devicePassword, devicepsw, strlen(devicepsw));
	}

//重定向
DESCIRB:
	if (!RecognVodVersion(m_sRtspUrl))
	{
		LOGD("CRtspClientEngine::PlaybackByTime RecognVersion fail\n");
		return HPR_TRUE;
	}

	HPR_BOOL bRet = ParseURL(m_sRtspUrl, deviceName, devicePassword);
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_RTSPURL_ERROR;
		LOGD("CRtspClientEngine::PlaybackByTime ParseURL Fail\n");
		return HPR_FALSE;
	}

	bRet = GenerateAuth(deviceName, devicePassword);
	if (!bRet) {
		LOGD("CRtspClientEngine::PlaybackByTime GenerateAuth Fail\n");
	}

	if (ConnectServer() == HPR_ERROR)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_CONNECT_SERVER_FAIL;
		LOGD("CRtspClientEngine::PlaybackByTime ConnectServer Fail\n");
		return HPR_FALSE;
	}

	bRet = Describe();
	if (!bRet)
	{
		if (m_statusCode == RTSP_STATUS_302 || m_statusCode == RTSP_STATUS_301)
		{
			bRet = GetRedirectUrl();
			if (bRet)
			{
				DisConnectServer();
				goto DESCIRB;
			}
		}

		DisConnectServer();
		LOGD("CRtspClientEngine::PlaybackByTime Describe Fail\n");
		return HPR_FALSE;
	}

	bRet = Setup();
	if (!bRet)
	{
		DisConnectServer();
		LOGD("CRtspClientEngine::PlaybackByTime Setup Fail\n");
		return HPR_FALSE;
	}

	bRet = Play(pfrom, pto);
	if (!bRet)
	{
		DisConnectServer();
		LOGD("CRtspClientEngine::PlaybackByTime Play Fail\n");
		return HPR_FALSE;
	}

	bRet = ChangeRate(NORMAL_SCALE);
	if(!bRet)
	{
		LOGD("CRtspClientEngine::PlaybackByTime ChangeRate Fail\n");
	}

	if (m_iVersion == VERSION_VTDU_4)
	{
		bRet = ForceIFrame();
		if(!bRet)
		{
			LOGD("CRtspClientEngine::PlaybackByTime ForceIFrame Fail\n");
		}
	}

	StartOptionsThread();

	m_bPlaying = HPR_TRUE;

	return HPR_TRUE;
}

// 在绝对时间范围内，随机定位
HPR_BOOL CRtspClientEngine::RandomPlay(pABS_TIME pfrom, pABS_TIME pto)
{
    if (m_state != RTSPSTATE_PLAY)
    {
        LOGD("CRtspClientEngine::ChanageBack not play back state\n");
        return HPR_FALSE;
    }

	if (m_sSendBuf == NULL || pfrom == NULL)
	{
		LOGD("CRtspClientEngine::Describe SendBuf is NULL\n");
		return HPR_FALSE;
	}

	RTSPCLIENT_PLAYPARAM param;
	memset(&param, 0, sizeof(RTSPCLIENT_PLAYPARAM));
	param.bhasPlayRange = true;            //需要"Range"字段
	param.timeType = RTSP_CLOCK_TIMETYPE;  //时间类型
	param.playTime = NULL;                 //开始播放时间
	param.bhasPlayScale = false;           //不需要"Scale"字段
	param.scale = 0;                       //播放倍率
	param.bBackward = false;               //表示倒放
	string starttime, endtime;
	if (HPR_FALSE == ConvertToStdTime(pfrom, starttime)) {
		LOGD("CRtspClientEngine::RandomPlay ConvertToStdTime fail\n");
		return HPR_FALSE;
	}
	else {
		param.startTime = (char*)starttime.c_str();
	}

	if(HPR_FALSE == ConvertToStdTime(pto, endtime))
	{
		param.endTime = NULL;
	}
	else
	{
		param.endTime = (char*)endtime.c_str();
	}

	HPR_BOOL bRet =  GeneratePlay(&param);
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_RANDOMPLAY_FAIL;
		LOGD("CRtspClientEngine::RandomPlay GeneratePlay fail\n");
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_RANDOMPLAY_FAIL;
		LOGD("CRtspClientEngine::RandomPlay SendRtspRequest fail\n");
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_RANDOMPLAY_FAIL;
			LOGD("CRtspClientEngine::RandomPlay RecvRtspResponse fail\n");
			return HPR_FALSE;
		}

		bRet = ParseRandomPlayPacket();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::RandomPlay ParseRandomPlayPacket fail\n");
			return HPR_FALSE;
		}
	}

	LOGD("CRtspClientEngine::RandomPlay success\n");
	return HPR_TRUE;
}

// 改变播放速率
HPR_BOOL CRtspClientEngine::ChangeRate(int scale)
{
	LOGD("CRtspClientEngine::ChanageRate scale:%d\n", scale);
    if (m_state != RTSPSTATE_PLAY)
    {

        LOGD("CRtspClientEngine::ChanageRate not play back state\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_PLAY_STATE;
        return HPR_FALSE;
    }

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Describe SendBuf is NULL\n");
		return HPR_FALSE;
	}

	 RTSPCLIENT_PLAYPARAM param;
	 memset(&param, 0, sizeof(RTSPCLIENT_PLAYPARAM));
	 param.bhasPlayRange = HPR_FALSE;   //是否带有Range头部
	 param.timeType = RTSP_NPT_TIMETYPE;//时间类型
	 param.startTime = NULL;            //开始时间
	 param.endTime = NULL;              //结束时间
	 param.playTime = NULL;             //开始播放时间
	 param.bhasPlayScale = HPR_TRUE;   //是否带有Scale头部
	 param.bBackward = HPR_FALSE;       //表示倒放
	 //wlf-2013-01-28 用于用于视频去 采用基线3.0的 8421-2-4-8(3210-1-2-3)
	/*if (scale >= 0)
	{
		playparam.scale = pow(RATE, scale);
	}
	else
	{
		playparam.scale = -pow(RATE, -scale);
	}*/
	// 各行业和流媒体4.0采用（84210.50.250.125）(3210-1-2-3)
	param.scale = pow(RATE, scale);   //播放倍率


	HPR_BOOL bRet =  GeneratePlay(&param);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::ChangeRate GeneratePlay fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_CHANGERATE_FAIL;
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::ChangeRate SendRtspRequest fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_CHANGERATE_FAIL;
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::ChangeRate RecvRtspResponse fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_CHANGERATE_FAIL;
			return HPR_FALSE;
		}

		bRet = ParseChanageRatePacket();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::ChangeRate ParseChanageRatePacket fail\n");
			return HPR_FALSE;
		}
	}

	if(m_state == RTSPSTATE_PAUSE) {

		if(m_iVersion == VERSION_VTDU_4)
		{
			PlayBackResume();
		}
		else
		{
			m_state = RTSPSTATE_PLAY;
		}

	}

	LOGD("CRtspClientEngine::ChangeRate success\n");
	return HPR_TRUE;
}

// 暂停回放
HPR_BOOL CRtspClientEngine::PlayBackPause()
{
	LOGD("enter CRtspClientEngine::PlayBackPause.");
	if (m_state != RTSPSTATE_PLAY)
	{
        LOGD("CRtspClientEngine::PlayBackPause not play back state\n");
        g_nRtspClientErrorCode = RTSPCLIENT_NO_PLAY_STATE;
        LOGD("leave CRtspClientEngine::PlayBackPause.");
		return HPR_FALSE;
	}

	// 生成Pause报文
	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::PlayBackPause SendBuf is NULL\n");
		LOGD("leave CRtspClientEngine::PlayBackPause.");
		return HPR_FALSE;
	}

	HPR_BOOL bRet = GeneratePause();
	if(!bRet)
	{
		LOGD("CRtspClientEngine::PlayBackPause GeneratePause fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_PAUSE_FAIL;
		LOGD("leave CRtspClientEngine::PlayBackPause.");
		return HPR_FALSE;
	}

	// 发送Pause报文
	bRet = SendRtspRequest();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_PAUSE_FAIL;
		LOGD("leave CRtspClientEngine::PlayBackPause.");
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		// 接收Pause报文
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_PAUSE_FAIL;
			LOGD("CRtspClientEngine::PlayBackPause RecvRtspResponse\n");
			LOGD("leave CRtspClientEngine::PlayBackPause.");
			return HPR_FALSE;
		}

		// 解析报文
		bRet = ParsePausePacket();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::PlayBackPause ParsePausePacket fail\n");
			LOGD("leave CRtspClientEngine::PlayBackPause.");
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_PAUSE;
	LOGD("CRtspClientEngine::PlayBackPause success\n");
	LOGD("leave CRtspClientEngine::PlayBackPause.");
	return HPR_TRUE;
}

// 恢复回放
HPR_BOOL CRtspClientEngine::PlayBackResume()
{
	if (m_state != RTSPSTATE_PAUSE)
	{
		LOGD("CRtspClientEngine::PlayBackResume not pause state\n");
		g_nRtspClientErrorCode = RTSPCLIENT_NOT_PAUSE_STATE_FAIL;
		return HPR_FALSE;
	}

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::PlayBackResume SendBuf is NULL\n");
		return HPR_FALSE;
	}

	HPR_BOOL bRet = GenerateResume();
	if(!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_RESUME_FAIL;
		LOGD("CRtspClientEngine::PlayBackResume GenerateResume fail\n");
		return HPR_FALSE;
	}

	// 发送Pause报文
	bRet = SendRtspRequest();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_RESUME_FAIL;
		LOGD("CRtspClientEngine::PlayBackResume SendRtspRequest fail\n");
		return HPR_FALSE;
	}


	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		// 接收Pause报文
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_RESUME_FAIL;
			LOGD("CRtspClientEngine::PlayBackResume RecvRtspResponse fail\n");
			return HPR_FALSE;
		}

		// 解析报文
		bRet = ParseResumePacket();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::PlayBackResume ParsePausePacket fail\n");
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_PLAY;
	LOGD("CRtspClientEngine::PlayBackResume ParsePausePacket success\n");
	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::MallocMemory()
{
    m_sSendBuf			= new char[RTSP_RTSPBUF_SIZE];
    m_sRecvBuf			= new char[RTSP_RTSPBUF_SIZE];
    m_sRtspUrl			= new char[MAX_URL_LEN];
    m_sLocationUrl		= new char[MAX_URL_LEN];
    m_sSession			= new char[RTSP_SESSION_SIZE];
    m_sTransportBuf		= new char[RTSP_TRANSPORT_SIZE];
    m_sServerPortBuf	= new char[RTSP_SERVERPORT_SIZE];
    m_sInterleavedBuf	= new char[RTSP_INTERLEAVE_SIZE];
    m_Auth 				= new char[MAX_AUTH_LEN];

    if (m_sSendBuf == NULL || m_sRecvBuf == NULL || m_sRtspUrl == NULL ||
        m_sLocationUrl == NULL ||m_sSession == NULL ||
        m_sTransportBuf == NULL || m_sServerPortBuf == NULL || m_sInterleavedBuf == NULL || m_Auth == NULL)
    {
    	LOGD("CRtspClientEngine::MallocMemory() fail");
        FreeMemory();
        return HPR_FALSE;
    }

    memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
    memset(m_sRecvBuf, 0, RTSP_RTSPBUF_SIZE);
    memset(m_sRtspUrl, 0, MAX_URL_LEN);
    memset(m_sLocationUrl, 0, MAX_URL_LEN);
    memset(m_sSession, 0, RTSP_SESSION_SIZE);
    memset(m_sTransportBuf, 0, RTSP_TRANSPORT_SIZE);
    memset(m_sServerPortBuf, 0, RTSP_SERVERPORT_SIZE);
    memset(m_sInterleavedBuf, 0, RTSP_INTERLEAVE_SIZE);
    memset(m_Auth, 0, MAX_AUTH_LEN);

    LOGD("CRtspClientEngine::MallocMemory() success");
    return HPR_TRUE;
}

void CRtspClientEngine::FreeMemory()
{
	if (m_sSendBuf != NULL)
	{
		delete[] m_sSendBuf;
		m_sSendBuf = NULL;
	}
	if (m_sRecvBuf != NULL)
	{
		delete[] m_sRecvBuf;
		m_sRecvBuf = NULL;
	}
	if (m_sRtspUrl != NULL)
	{
		delete[] m_sRtspUrl;
		m_sRtspUrl = NULL;
	}
	if (m_sLocationUrl != NULL)
	{
		delete[] m_sLocationUrl;
		m_sLocationUrl = NULL;
	}
	if (m_sSession != NULL)
	{
		delete[] m_sSession;
		m_sSession = NULL;
	}
	if (m_sTransportBuf != NULL)
	{
		delete[] m_sTransportBuf;
		m_sTransportBuf = NULL;
	}
	if (m_sServerPortBuf != NULL)
	{
		delete[] m_sServerPortBuf;
		m_sServerPortBuf = NULL;
	}
	if (m_sInterleavedBuf != NULL)
	{
		delete[] m_sInterleavedBuf;
		m_sInterleavedBuf = NULL;
	}
	if (m_Auth != NULL)
	{
		delete[] m_Auth;
		m_Auth = NULL;
	}

	LOGD("CRtspClientEngine::FreeMemory() success");
}

HPR_BOOL CRtspClientEngine::RecognRealVersion(const char* pRtspURL)
{
	if (pRtspURL == NULL)
	{
		LOGD("CRtspClientEngine::RecognRealVersion rtsp url is null");
		return HPR_FALSE;
	}

	while (true)
	{
		char* keyPos = NULL;

		// 流媒体2.0
		keyPos = HPR_Strcasestr(pRtspURL, "livestream");
		if (keyPos)
		{
			m_iVersion = VERSION_MAG;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			m_UserAgent = "HIKRTSPClient/1.0";
			LOGD("CRtspClientEngine::RecognRealVersion MAG 2.0");
			break;
		}

		// 流媒体4.0
		/*keyPos = HPR_Strcasestr(pRtspURL, "hikvision");
		if (keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			m_UserAgent = "PhoneClient";
			LOGD("CRtspClientEngine::RecognRealVersion hikvision VTDU 4.0");
			break;
		}

		// 最新MAG的地址
		keyPos = HPR_Strcasestr(pRtspURL, "realplay");
		if(keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			m_UserAgent = "MCU";
			LOGD("CRtspClientEngine::RecognRealVersion realplay new mag");
			break;
		}

		// 最新MAG的地址
		keyPos = HPR_Strcasestr(pRtspURL, "ncg");
		if(keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			m_UserAgent = "MCU";
			LOGD("CRtspClientEngine::RecognRealVersion realplay new mag");
			break;
		}

//		keyPos = HPR_Strcasestr(pRtspURL, "pag");
//		if(keyPos)
//		{
//			m_iVersion = VERSION_VTDU_4;
//			m_iRtpProtocol = RTPRTSP_TRANSMODE;
//			m_UserAgent = "MCU";
//			LOGD("CRtspClientEngine::RecognRealVersion pag ");
//			break;
//		}

		keyPos = HPR_Strcasestr(pRtspURL, "IAS");
		if(keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			m_UserAgent = "MCU";
			LOGD("CRtspClientEngine::RecognRealVersion IAS ");
			break;
		}

		// 默认流媒体3.0
		m_iVersion = VERSION_VTDU_3;
		m_iRtpProtocol = RTPTCP_TRANSMODE;*/

		// 流媒体3.0
		keyPos = HPR_Strcasestr(pRtspURL, "hc8");
		if (keyPos)
		{
			m_iVersion = VERSION_VTDU_3;
			m_iRtpProtocol = RTPTCP_TRANSMODE;
			m_UserAgent = "MCU";
			LOGD("CRtspClientEngine::RecognRealVersion hc8 ");
			break;
		}

		// 默认流媒体4.0
		m_iVersion = VERSION_VTDU_4;
		m_iRtpProtocol = RTPRTSP_TRANSMODE;
		m_UserAgent = "MCU";
		break;
	}

	if (m_iVersion != VERSION_VTDU_3 && m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_VERSION_PROTOCOL_NOTSUPPORT;
		LOGD("CRtspClientEngine::RecognRealVersion MAG and VTDU 4.0 no support Rtp or Tcp transmode");
		return HPR_FALSE;
	}

	LOGI("CRtspClientEngine::RecognRealVersion success version = %d", m_iVersion);
	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::RecognVodVersion(const char* pRtspURL)
{
	if (pRtspURL == NULL)
	{
		LOGD("CRtspClientEngine::RecognVodVersion rtsp url is null");
		return HPR_FALSE;
	}

	while (true)
	{
		char* keyPos = NULL;
		keyPos = HPR_Strcasestr(pRtspURL, "vod");
		if (keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			LOGD("CRtspClientEngine::RecognVodVersion vod ");
			break;
		}


		keyPos = HPR_Strcasestr(pRtspURL, "file");
		if (keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			LOGD("CRtspClientEngine::RecognVodVersion file");
			break;
		}

		// 增加级联回放URL关键字判断 add by hyf 20160627
		keyPos = HPR_Strcasestr(pRtspURL, "ncg_playback");
		if (keyPos)
		{
			m_iVersion = VERSION_VTDU_4;
			m_iRtpProtocol = RTPRTSP_TRANSMODE;
			LOGD("CRtspClientEngine::RecognVodVersion ncg_playback");
			break;
		}

		m_iVersion = VERSION_VTDU_3;
		m_iRtpProtocol = RTPTCP_TRANSMODE;
		break;
	}


	if (m_iVersion != VERSION_VTDU_3 && m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_VERSION_PROTOCOL_NOTSUPPORT;
		LOGD("CRtspClientEngine::RecognVodVersion MAG and VTDU 4.0 no support Rtp or Tcp transmode");
		return HPR_FALSE;
	}

	LOGD("CRtspClientEngine::RecognVodVersion version = %d", m_iVersion);

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseURL(char* pRtspURL, char* deviceName, char* devicePassword)
{
	if (pRtspURL == NULL) {
		LOGD("CRtspClientEngine::ParseURL rtsp url is null");
		return HPR_FALSE;
	}

	int iLen = strlen(pRtspURL);
	if (iLen < 16 || iLen > MAX_URL_LEN)
	{
		LOGD("CRtspClientEngine::ParseURL() rtsp url len < 16");
		return HPR_FALSE;
	}

	if (strncmp(pRtspURL, "rtsp://", 7) != 0)
	{
		LOGD("CRtspClientEngine::ParseURL() rtsp url don't start with rtsp\n");
		return HPR_FALSE;
	}

	char* pos4 = NULL;

	char* pos1 = NULL;
	char* pos2 = NULL;
	char* pos3 = NULL;
	bool bPort = true;

	char* pos5 = NULL;
	char* pos6 = NULL;

	pos1 = (char*)pRtspURL + 7;
	pos4 = strstr(pos1, "@");
	if (pos4) {
		LOGD(pos4);
		// 解析出用户名和密码并修改取流地址
		pos5 = strstr(pos1, ":");
		if (pos4 > pos5) {
			memset(deviceName, 0, MAX_USERNAME_LEN);
			memcpy(deviceName, pos1, pos5-pos1);
			memset(devicePassword, 0, MAX_USERNAME_LEN);
			memcpy(devicePassword, pos5+1, pos4-pos5-1);
		}
		memcpy(pRtspURL + 7, pos4 + 1, (strlen(pRtspURL) - 7 - (pos4 - pos1)));
		pos1 = pRtspURL + 7;
	}

	pos2 = strstr(pos1, ":");
	pos3 = strstr(pos1, "/");
	if (!pos2 || pos3 <= pos2)
	{
		pos2 = pos3;
		bPort = false;
	}

	if (!pos3) {
		LOGD("CRtspClientEngine::ParseURL() don't find '/' after rtsp://\n");
		return HPR_FALSE;
	}

	int addrLen = pos2 - pos1;
	if (addrLen < 3)
	{
		LOGD("CRtspClientEngine::ParseURL() addrLen len is invalide\n");
		return HPR_FALSE;
	}
	char strAddr[addrLen + 1];
	memset(strAddr, 0, addrLen + 1);
	memcpy((char*)strAddr, pos1, addrLen);
	strAddr[addrLen] = '\0';
	LOGD("CRtspClientEngine::ParseURL() addr is [%s]", strAddr);

	// 域名解析
#ifdef HK_APPLE
    memset(&m_sRTSPServerAddr, 0, IPADDR_LEN);
    hostent *addrHostent = HPR_GetAddressByName(strAddr);
    if (NULL != addrHostent)
    {
        if (inet_ntop(addrHostent->h_addrtype, (const void *)*addrHostent->h_addr_list, (char *)m_sRTSPServerAddr, INET_ADDRSTRLEN) == NULL)
        {
            CMCRPrint::Print("CRtspClientEngine::ParseURL() HPR_GetAddressByName is failed\n");
            return HPR_FALSE;
        }
    }
    else
    {
        memcpy(&m_sRTSPServerAddr, strAddr, addrLen);
    }
    CMCRPrint::Print((const char *)m_sRTSPServerAddr);
    m_iRTSPServerAddr = inet_addr((char*)m_sRTSPServerAddr);
#else
	hostent *addrHostent = gethostbyname(strAddr);
	//hostent *addrHostent = HPR_GetAddressByName(strAddr);
	if (NULL != addrHostent)
	{
		char ip[16];
		memset(ip, 0, 16);
        if (inet_ntop(addrHostent->h_addrtype, (const void *)*addrHostent->h_addr_list, (char *)ip, INET_ADDRSTRLEN) == NULL)
        {
            LOGD("CRtspClientEngine::ParseURL() HPR_GetAddressByName is failed\n");
            return HPR_FALSE;
        }
        m_iRTSPServerAddr = inet_addr((char*)ip);
        LOGD("CRtspClientEngine::ParseURL()gethostbyname ip is[%s]\n", ip);
	}
	else
	{
		m_iRTSPServerAddr = inet_addr((char*)strAddr);
		LOGD("CRtspClientEngine::ParseURL()ip is[%s]\n", strAddr);
	}
#endif
	if (bPort)
	{
		char sPort[16];
		memset(sPort, 0, 16);
		if (pos3 - pos2 > 15)
		{
			LOGD("CRtspClientEngine::ParseURL() port len > 15\n");
			return HPR_FALSE;
		}
		memcpy((char*)sPort, pos2 + 1, pos3 - pos2 - 1);
		LOGD(sPort);
		m_iRTSPServerPort = HPR_Atoi32(sPort);
	}
	else
	{
		m_iRTSPServerPort = 554;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GenerateAuth(const char* devicename, const char* devicepsw)
{
	if (m_iVersion == VERSION_MAG)
	{
		 LOGD("CRtspParser::GenerateAuth() version is 2, no need name and password\n");
		return HPR_TRUE;
	}

    if (NULL == devicename || strlen(devicename) == 0
    		|| NULL == devicepsw || strlen(devicepsw) == 0)
    {
    	LOGD("no Authorization.");
    	bzero(m_Auth, strlen(m_Auth));
    	return HPR_TRUE;
    }

    HPR_UINT32 usernamelen = (HPR_UINT32)strlen(devicename);
    HPR_UINT32 passwdlen = (HPR_UINT32)strlen(devicepsw);
    if (usernamelen > MAX_USERNAME_LEN ||  passwdlen > MAX_PASSWD_LEN)
    {
    	 LOGD("CRtspParser::GenerateAuth() devicename or password len is incorrect\n");
        return HPR_FALSE;
    }

    if (usernamelen + passwdlen + 1> MAX_AUTH_LEN)
    {
    	 LOGD("CRtspParser::GenerateAuth() devicename and password len is incorrect\n");
        return HPR_FALSE;
    }
//    LOGD("deviceName:%s", devicename);
//    LOGD("devicepsw:%s", devicepsw);//信息安全不能打印密码，并且iOS编译出错
    if (NULL == devicename || strlen(devicename) == 0
    		|| NULL == devicepsw || strlen(devicepsw) == 0)
    {
    	LOGD("no Authorization.");
    	bzero(m_Auth, strlen(m_Auth));
    	return HPR_TRUE;
    }
    char authbef[MAX_AUTH_LEN + 1] = {0};
    sprintf(authbef, "%s:%s", devicename, devicepsw);

    char encbuf[1024];
    HPR_INT32 nRet = encode_base64((HPR_INT32)strlen(authbef), (unsigned char *)authbef,(unsigned char*)encbuf);
    if (nRet != HPR_OK)
    {
    	 LOGD("CRtspParser::GenerateAuth() encode_base64 fail\n");
        return HPR_FALSE;
    }

    HPR_INT32 encauthlen = (HPR_INT32)strlen(encbuf);
    if (encauthlen > 0)
    {
        const char *format = "Basic %s";
        HPR_INT32 formatlen = (HPR_INT32)strlen(format);
        if (formatlen + encauthlen > MAX_AUTH_LEN)
        {
        	 LOGD("CRtspParser::GenerateAuth() encode len incorrect\n");
            return HPR_FALSE;
        }
        sprintf(m_Auth, format, encbuf);
    }
    else
    {
    	 LOGD("CRtspParser::GenerateAuth() encauthlen < 0\n");
        return HPR_FALSE;
    }

    LOGD("CRtspParser::GenerateAuth() success\n");
    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ConnectServer()
{
#ifdef HK_APPLE
    CMCRPrint::Print("CRtspClientEngine::ConnecServer()\n");
    HPR_ADDRESS_INFO *pAddressInfo = NULL;
    if ((HPR_GetAddressInfo((const char *)m_sRTSPServerAddr, NULL, NULL, &pAddressInfo) != 0))
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() HPR_GetAddressInfo is failed\n");
        return HPR_FALSE;
    }
    
    if (NULL == pAddressInfo)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() pAddressInfo is NULL\n");
        return HPR_FALSE;
    }
    
    HPR_ADDRESS_INFO *pAddrTmp = pAddressInfo;
    char szIPBUf[128] = {0};
    while (pAddrTmp != NULL)
    {
        HPR_GetAddrStringEx(&(pAddrTmp->Address), szIPBUf, 128);
        // 开启ipv6解析域名，会出现回路地址：：1，过滤掉
        if (0 != strcmp(szIPBUf, "::1"))
        {
            break;
        }
        else
        {
            pAddrTmp = pAddrTmp->pNext;
        }
    }
    
    CMCRPrint::Print("CRtspClientEngine::ConnectServer() ip:[%s]\n", szIPBUf);
    
    bool bIPIsV6 = false;
    if (strstr(szIPBUf, ":") != NULL)
    {
        bIPIsV6 = true;
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() It is ipv6 address\n");
    }
    else
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() It is ipv4 address\n");
    }
    
    if (!bIPIsV6 && strlen(szIPBUf) > 16)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() ipV4 addr is invalide\n");
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    
    HPR_INT32 iAF = bIPIsV6 ? AF_INET6 : AF_INET;
    m_iRTSPSocket = HPR_CreateSocket(iAF, SOCK_STREAM, IPPROTO_TCP);
    if (m_iRTSPSocket == HPR_INVALID_SOCKET)
    {
        CMCRPrint::Print("CRtspClientEngine[%d]::ConnectServer() create socket fail, errorCode:%u and system errorCode:%u\n", \
                         m_iEngineerIndex, \
                         HPR_GetLastError(), \
                         HPR_GetSystemLastError());
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    
    // 设置套接字接收缓冲大小和发送缓冲大小
    HPR_INT32 ret = HPR_SetBuffSize(m_iRTSPSocket, 5*RTP_RTPBUF_SIZE, 5*RTP_RTPBUF_SIZE);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() HPR_SetBuffSize fail, errorCode:%u and system errorCode:%u\n", \
                         m_iEngineerIndex, \
                         HPR_GetLastError(), \
                         HPR_GetSystemLastError());
        
        HPR_CloseSocket(m_iRTSPSocket);
        m_iRTSPSocket = HPR_INVALID_SOCKET;
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    
    // 设置地址重用
    ret = HPR_SetReuseAddr(m_iRTSPSocket, true);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() HPR_SetReuseAddr fail, errorCode:%u and system errorCode:%u\n", \
                         m_iEngineerIndex, \
                         HPR_GetLastError(), \
                         HPR_GetSystemLastError());
        
        HPR_CloseSocket(m_iRTSPSocket);
        m_iRTSPSocket = HPR_INVALID_SOCKET;
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    
    // 连接服务器，超时时间是毫秒,超时时间改为10s
    pAddressInfo->Address.SA.sin4.sin_port = htons(m_iRTSPServerPort);
    pAddressInfo->Address.SA.sin6.sin6_port = htons(m_iRTSPServerPort);
    ret = HPR_ConnectWithTimeOut(m_iRTSPSocket, &pAddressInfo->Address, 10000);
    if (ret != HPR_OK)
    {
        CMCRPrint::Print("CRtspClientEngine::ConnectServer() connect fail, last error code:%d and system error code:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
        HPR_CloseSocket(m_iRTSPSocket);
        m_iRTSPSocket = HPR_INVALID_SOCKET;
        HPR_FreeAddressInfo(pAddressInfo);
        return HPR_ERROR;
    }
    HPR_FreeAddressInfo(pAddressInfo);
    CMCRPrint::Print("CRtspClientEngine::ConnectServer() success, errorCode:%u and system errorCode:%u\n", \
                     m_iEngineerIndex, \
                     HPR_GetLastError(), \
                     HPR_GetSystemLastError());
    
    return HPR_OK;
#else
    LOGD("CRtspClientEngine::ConnecServer()\n");

	m_iRTSPSocket = HPR_CreateSocket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(m_iRTSPSocket == HPR_INVALID_SOCKET) {
		LOGD("CRtspClientEngine[%d]::ConnectServer() create socket fail, errorCode:%u and system errorCode:%u\n", \
				m_iEngineerIndex, \
				HPR_GetLastError(), \
				HPR_GetSystemLastError());
		return HPR_ERROR;
	}

	// 设置套接字接收缓冲大小和发送缓冲大小
	HPR_INT32 ret = HPR_SetBuffSize(m_iRTSPSocket, 5*RTP_RTPBUF_SIZE, 5*RTP_RTPBUF_SIZE);
	if (ret != HPR_OK)
	{
		LOGD("CRtspClientEngine::ConnectServer() HPR_SetBuffSize fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

		HPR_CloseSocket(m_iRTSPSocket);
		m_iRTSPSocket = HPR_INVALID_SOCKET;
		return HPR_ERROR;
	}

	// 设置地址重用
	ret = HPR_SetReuseAddr(m_iRTSPSocket, true);
	if (ret != HPR_OK)
	{
		LOGD("CRtspClientEngine::ConnectServer() HPR_SetReuseAddr fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

		HPR_CloseSocket(m_iRTSPSocket);
		m_iRTSPSocket = HPR_INVALID_SOCKET;
		return HPR_ERROR;
	}

	// 通过地址族类型,字符串格式的地址及端口号组建HPR网络地址结构
	HPR_ADDR_T remoteaddr;
	memset(&remoteaddr, 0, sizeof(remoteaddr));
	ret = HPR_MakeAddr4ByInt(m_iRTSPServerAddr, m_iRTSPServerPort, &remoteaddr);
	if (ret != HPR_OK)
	{
		LOGD("CRtspClientEngine::ConnectServer() HPR_MakeAddrByString fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
		HPR_CloseSocket(m_iRTSPSocket);
		m_iRTSPSocket = HPR_INVALID_SOCKET;
		return HPR_ERROR;
	}

	// 连接服务器，超时时间是毫秒
	ret = HPR_ConnectWithTimeOut(m_iRTSPSocket, &remoteaddr, 20000);
	if (ret != HPR_OK)
	{
		LOGD("CRtspClientEngine::ConnectServer() connect fail, last error code:%d and system error code:%d\n", HPR_GetLastError(), HPR_GetSystemLastError());
		HPR_CloseSocket(m_iRTSPSocket);
		m_iRTSPSocket = HPR_INVALID_SOCKET;
		return HPR_ERROR;
	}

	LOGD("CRtspClientEngine::ConnectServer() success, errorCode:%u and system errorCode:%u\n", \
							m_iEngineerIndex, \
							HPR_GetLastError(), \
							HPR_GetSystemLastError());

	return HPR_OK;
#endif
}

HPR_BOOL CRtspClientEngine::Describe()
{
	LOGD("CRtspClientEngine::Describe()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Describe SendBuf is NULL\n");
		return HPR_FALSE;
	}

	RTSPCLIENT_DESCRIBEPARAM describeparam;
	memset(&describeparam, 0, sizeof(RTSPCLIENT_DESCRIBEPARAM));

	char acceptptn[MAX_ACCEPTTYEP_LEN + 1];
	memset(acceptptn, 0, MAX_ACCEPTTYEP_LEN + 1);
	strcpy(acceptptn, "application/sdp");
	char *acceptpointarray[1];
	acceptpointarray[0] = acceptptn;
	describeparam.acceptpresentation = acceptpointarray;
	describeparam.presentationcount = 1;
	describeparam.bstandardstream = false;
	describeparam.bdevicepushdata = false;
	describeparam.bdownload = false;

	HPR_BOOL bRet =  GenerateDescribe(&describeparam);
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_DESCRIBE_FAIL;
		LOGD("CRtspClientEngine::Describe GenerateDescribe fail\n");
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_DESCRIBE_FAIL;
		LOGD("CRtspClientEngine::Describe Send Describe fail\n");
		return HPR_FALSE;
	}

	bRet = RecvRtspResponse();
	if (!bRet)
	{
		if (!m_bVTDUStackErr)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_DESCRIBE_FAIL;
		}
		LOGD("CRtspClientEngine::Describe recv Describe fail\n");
		return HPR_FALSE;
	}

	bRet = ParseDescribePacket();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Describe ParseDescribePacket fail\n");
		return HPR_FALSE;
	}

	m_state = RTSPSTATE_DESCRIBE;

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::Setup()
{
	LOGD("CRtspClientEngine::Setup()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Describe SendBuf is NULL\n");
		return HPR_FALSE;
	}

	if (m_oSdpInfo.mediacount > 0)
	{
		for (HPR_INT32 i = 0; i < m_oSdpInfo.mediacount; i++)
		{
			RTSPCLIENT_SETUPPARAM setupparam;
			setupparam.url = m_oSdpInfo.mediaurl[i];
		    setupparam.medieIndex = i;
			HPR_BOOL bRet =  GenerateSetup(&setupparam);
			if (!bRet)
			{
				g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_SETUP_FAIL;
				LOGD("CRtspClientEngine::Setup Generate Setup fail\n");
				return HPR_FALSE;
			}

			bRet = SendRtspRequest();
			if (!bRet)
			{
				g_nRtspClientErrorCode = RTSPCLIENT_SEND_SETUP_FAIL;
				LOGD("CRtspClientEngine::Setup SendRtspRequest fail\n");
				return HPR_FALSE;
			}

			bRet = RecvRtspResponse();
			if (!bRet)
			{
				if (!m_bVTDUStackErr)
				{
					g_nRtspClientErrorCode = RTSPCLIENT_RECV_SETUP_FAIL;
				}
				LOGD("CRtspClientEngine::Setup RecvRtspResponse fail\n");
				return HPR_FALSE;
			}

			bRet = ParseSetupPacket(i);
			if (!bRet)
			{
				return HPR_FALSE;
			}

			break;
		}
	}
	else
	{
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		bool bRet = StartRtpTcpTrans();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::Setup StartRtpTcpTrans fail\n");
			return HPR_FALSE;
		}
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE)
	{
		LOGD("CRtspClientEngine::Setup start rtp or udp.....\n");
		bool bRet = StartRtpUdpTrans();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::Setup StartRtpUdpTrans fail\n");
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_SETUP;

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::Play()
{
	LOGD("CRtspClientEngine::Play()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Play m_sSendBuf is null\n");
		return HPR_FALSE;
	}
	RTSPCLIENT_PLAYPARAM playparam;
	memset(&playparam, 0, sizeof(RTSPCLIENT_PLAYPARAM));
	playparam.bhasPlayRange = true;            //需要"Range"字段
	playparam.timeType = RTSP_NPT_TIMETYPE;    //时间类型
	playparam.playTime = NULL;                 //开始播放时间
	playparam.bhasPlayScale = false;           //不需要"Scale"字段
	playparam.scale = 0;                       //播放倍率
	playparam.bBackward = false;               //表示倒放
	playparam.endTime = NULL;
	playparam.startTime = NULL;

	HPR_BOOL bRet =  GeneratePlay(&playparam);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play GeneratePlay fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_PLAY_FAIL;
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play SendRtspRequest fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_PLAY_FAIL;
		return HPR_FALSE;
	}

	bRet = RecvRtspResponse();
	if (!bRet)
	{
		if (!m_bVTDUStackErr)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_PLAY_FAIL;
		}
		LOGD("CRtspClientEngine::Play RecvRtspResponse fail\n");
		return HPR_FALSE;
	}

	bRet = ParsePlayPacket();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play ParsePlayPacket fail\n");
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPRTSP_TRANSMODE)
	{
		bRet = StartRtpRtspTrans();
		if (!bRet)
		{
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_PLAY;

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::Play(pABS_TIME pfrom, pABS_TIME pto)
{
	LOGD("CRtspClientEngine::Play()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Describe SendBuf is NULL\n");
		return HPR_FALSE;
	}

	RTSPCLIENT_PLAYPARAM playparam;
	memset(&playparam, 0, sizeof(RTSPCLIENT_PLAYPARAM));
	playparam.bhasPlayRange = true;            //需要"Range"字段
	playparam.timeType = RTSP_CLOCK_TIMETYPE;  //时间类型
	playparam.playTime = NULL;                 //开始播放时间
	playparam.bhasPlayScale = false;           //不需要"Scale"字段
	playparam.scale = 0;                       //播放倍率
	playparam.bBackward = false;               //表示倒放
	string starttime, endtime;
	if (!ConvertToStdTime(pfrom, starttime)) {
		return HPR_FALSE;
	}
	else {
		playparam.startTime = (char*)starttime.c_str();
	}

	if(!ConvertToStdTime(pto, endtime))
	{
		playparam.endTime = NULL;
	}
	else
	{
		playparam.endTime = (char*)endtime.c_str();
	}

	HPR_BOOL bRet =  GeneratePlay(&playparam);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play GeneratePlay fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_PLAY_FAIL;
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play SendRtspRequest fail\n");
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_PLAY_FAIL;
		return HPR_FALSE;
	}

	bRet = RecvRtspResponse();
	if (!bRet)
	{
		if (!m_bVTDUStackErr)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RECV_PLAY_FAIL;
		}
		LOGD("CRtspClientEngine::Play RecvRtspResponse fail\n");
		return HPR_FALSE;
	}

	bRet = ParsePlayPacket();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::Play ParsePlayPacket fail\n");
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPRTSP_TRANSMODE)
	{
		bRet = StartRtpRtspTrans();
		if (!bRet)
		{
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_PLAY;

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ForceIFrame()
{
	LOGD("CRtspClientEngine::ForceIFrame()\n");

	// 生成ForceIFrame报文
	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::ForceIFrame SendBuf is NULL\n");
		return HPR_FALSE;
	}

	HPR_BOOL bRet = GenerateForceIFrame();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_GENERATE_FORCEIFRAME_FAIL;
		LOGD("CRtspClientEngine::ForceIFrame SendRtspRequest fail\n");
		return HPR_FALSE;
	}

	// 发送ForceIFrame报文
	bRet = SendRtspRequest();
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_SEND_FORCEIFRAME_FAIL;
		LOGD("CRtspClientEngine::ForceIFrame SendRtspRequest fail\n");
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		//接收ForceIFrame报文
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			if (!m_bVTDUStackErr)
			{
				g_nRtspClientErrorCode = RTSPCLIENT_RECV_FORCEIFRAME_FAIL;
			}
			LOGD("CRtspClientEngine::ForceIFrame RecvRtspResponse fail\n");
			return HPR_FALSE;
		}

		// 解析报文
		bRet = ParseForceIFramePacket();
		if (!bRet)
		{
			LOGD("CRtspClientEngine::ForceIFrame ParseForceIFramePacket fail\n");
			return HPR_FALSE;
		}
	}

	return HPR_TRUE;
}

int CRtspClientEngine::TearDown()
{
	LOGD("CRtspClientEngine::TearDown()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::RandomPlay SendBuf is NULL\n");
		return HPR_FALSE;
	}

	HPR_BOOL bRet = GenerateTearDown();
	if (!bRet)
	{
		return HPR_FALSE;
	}

	bRet = SendRtspRequest();
	if (!bRet)
	{
		return HPR_FALSE;
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		bRet = RecvRtspResponse();
		if (!bRet)
		{
			return HPR_FALSE;
		}

		bRet = ParseTearDownPacket();
		if (!bRet)
		{
			return HPR_FALSE;
		}
	}

	m_state = RTSPSTATE_TEARDOWN;
	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GenerateDescribe(PRTSPCLIENT_DESCRIBEPARAM describeparam) {
	if (m_sSendBuf == NULL || describeparam == NULL) {
		LOGD("CRtspClientEngine::Describe m_sSendBuf or param is NULL\n");
		return HPR_FALSE;
	}

	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	strcpy(m_sSendBuf, "DESCRIBE ");
	strcat(m_sSendBuf, m_sRtspUrl);
	strcat(m_sSendBuf, " RTSP/1.0\r\n");
	sprintf(m_sSendBuf + strlen(m_sSendBuf), "CSeq:%d\r\n", m_iSeq++);
	strcat(m_sSendBuf, "Accept:");
	for (HPR_INT32 i = 0; i < describeparam->presentationcount; i++)
	{
		strcat(m_sSendBuf, describeparam->acceptpresentation[i]);
		if (i + 1 == describeparam->presentationcount)
		{
			break;
		}
		strcat(m_sSendBuf, ",");
	}
	strcat(m_sSendBuf, "\r\n");

	HPR_INT32 authlen = (HPR_INT32)strlen(m_Auth);
	if (authlen)
	{
		strcat(m_sSendBuf, "Authorization:");
		strcat(m_sSendBuf, m_Auth);
		strcat(m_sSendBuf, "\r\n");
	}

	if (m_iVersion == VERSION_VTDU_3)
	{
		if (describeparam->bstandardstream == true)
		{
			strcat(m_sSendBuf, "StandardStream:1\r\n");
		}
		else
		{
			strcat(m_sSendBuf, "StandardStream:0\r\n");
		}

		if (describeparam->bdevicepushdata == true)
		{
			strcat(m_sSendBuf, "Device-PushData:1\r\n");
		}
		else
		{
			strcat(m_sSendBuf, "Device-PushData:0\r\n");
		}

		if (describeparam->bdownload == true)
		{
			strcat(m_sSendBuf, "Download:1\r\n");
		}
		else
		{
			strcat(m_sSendBuf, "Download:0\r\n");
		}
	}

	strcat(m_sSendBuf, "User-Agent:");
	strcat(m_sSendBuf, m_UserAgent);
	strcat(m_sSendBuf, "\r\n\r\n");

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GenerateSetup(PRTSPCLIENT_SETUPPARAM setupparam) {
	if (m_sSendBuf == NULL || setupparam == NULL) {
		return HPR_FALSE;
	}

	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	strcpy(m_sSendBuf, "SETUP ");
	strcat(m_sSendBuf, setupparam->url);
	strcat(m_sSendBuf, " RTSP/1.0\r\n");
	sprintf(m_sSendBuf + strlen(m_sSendBuf), "CSeq:%d\r\n", m_iSeq++);

	if (m_iRtpProtocol == RTPRTSP_TRANSMODE)			//RTP/RTSP
	{
		sprintf(m_sSendBuf + strlen(m_sSendBuf), "Transport:RTP/AVP/TCP;unicast;interleaved=%u-%u;ssrc=%u\r\n", 0, 1, 0);
	}
	else if (m_iRtpProtocol == RTPTCP_TRANSMODE)		//RTP/TCP
	{
		if (m_oUdpTcpPortPair[setupparam->medieIndex].GetRTPClientPort() <= 0)
		{
			return HPR_FALSE;
		}
		sprintf(m_sSendBuf + strlen(m_sSendBuf), "Transport:RTP/AVP/TCP;unicast;client_port=%u-%u;ssrc=%u\r\n",\
				m_oUdpTcpPortPair[setupparam->medieIndex].GetRTPClientPort(), m_oUdpTcpPortPair[setupparam->medieIndex].GetRTCPClientPort(), 0);
	}
	else if (m_iRtpProtocol == RTPUDP_TRANSMODE)		//RTP/UDP
	{
		if (m_oUdpTcpPortPair[setupparam->medieIndex].GetRTPClientPort() <= 0)
		{
			return HPR_FALSE;
		}
		if (m_bretrans)
		{
			sprintf(m_sSendBuf + strlen(m_sSendBuf), "Retrans:%u\r\n", 1);
		}
		sprintf(m_sSendBuf + strlen(m_sSendBuf), "Transport:RTP/AVP/UDP;unicast;client_port=%u-%u;ssrc=%u\r\n",\
				m_oUdpTcpPortPair[setupparam->medieIndex].GetRTPClientPort(), m_oUdpTcpPortPair[setupparam->medieIndex].GetRTCPClientPort(), 0);
	}
	else if (m_iRtpProtocol == RTPMCAST_TRANSMODE)		//RTP/multicast
	{
		sprintf(m_sSendBuf + strlen(m_sSendBuf), "Transport:RTP/AVP/UDP;multicast;ssrc=%u\r\n", 0);
	}
	else
	{
		return HPR_FALSE;
	}

	strcat(m_sSendBuf, "User-Agent:");
	strcat(m_sSendBuf, m_UserAgent);
	strcat(m_sSendBuf, "\r\n\r\n");

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GeneratePlay(PRTSPCLIENT_PLAYPARAM playparam) {

	if (m_sSendBuf == NULL || playparam == NULL) {
		LOGD("CRtspClientEngine::GeneratePlay m_sSendBuf or param is null\n");
		return HPR_FALSE;
	}

	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	strcpy(m_sSendBuf, "PLAY ");
	strcat(m_sSendBuf, m_sRtspUrl);
	strcat(m_sSendBuf, " RTSP/1.0\r\n");
	sprintf(m_sSendBuf + strlen(m_sSendBuf), "CSeq:%d\r\n", m_iSeq++);
	sprintf(m_sSendBuf + strlen(m_sSendBuf), "Session:%s\r\n", m_sSession);
	if (playparam->bhasPlayRange)
	{
		if (playparam->timeType == RTSP_NPT_TIMETYPE)
		{
			strcat(m_sSendBuf, "Range:npt=");
			if (playparam->startTime)
			{
				strcat(m_sSendBuf, playparam->startTime);
			}
			else
			{
				strcat(m_sSendBuf, "now");
			}

			strcat(m_sSendBuf, "-");
			if (playparam->endTime)
			{
				strcat(m_sSendBuf, playparam->endTime);
			}
			strcat(m_sSendBuf, "\r\n");
		}
		else if (playparam->timeType == RTSP_SMPTE_TIMETYPE)
		{
			LOGD("CRtspClientEngine::GeneratePlay not support RTSP_SMPTE_TIMETYPE \n");
			return HPR_FALSE;
		}
		else if (playparam->timeType == RTSP_CLOCK_TIMETYPE)
		{
			strcat(m_sSendBuf, "Range:clock=");
			if(playparam->startTime)
			{
				strcat(m_sSendBuf, playparam->startTime);
			}
			else
			{
				LOGD("CRtspClientEngine::GeneratePlay in RTSP_CLOCK_TIMETYPE, start time is null\n");
				return HPR_FALSE;
			}

			strcat(m_sSendBuf, "-");
			if(playparam->endTime)
			{
				strcat(m_sSendBuf, playparam->endTime);
			}
			strcat(m_sSendBuf, "\r\n");
		}
		else
		{
			LOGD("CRtspClientEngine::GeneratePlay unkown time type\n");
			return HPR_FALSE;
		}
	}

	if (playparam->bhasPlayScale)
	{
		sprintf(m_sSendBuf + strlen(m_sSendBuf), "Scale:%.6g\r\n", playparam->scale);
	}

	if (playparam->bBackward)
	{
		strcat(m_sSendBuf, "Backward:clock=");
		if(playparam->startTime)
		{
			strcat(m_sSendBuf, playparam->startTime);
		}
		else
		{
			LOGD("CRtspClientEngine::GeneratePlay int bBackward, start time is null \n");
			return HPR_FALSE;
		}

		strcat(m_sSendBuf, "-");
		if(playparam->endTime)
		{
			strcat(m_sSendBuf, playparam->endTime);
		}
		strcat(m_sSendBuf, "\r\n");
	}

	strcat(m_sSendBuf, "User-Agent:");
	strcat(m_sSendBuf, m_UserAgent);
	strcat(m_sSendBuf, "\r\n\r\n");

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GenerateForceIFrame()
{
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}

	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);

	if (m_iVersion == VERSION_VTDU_3)
	{
		const char* format = "OPTIONS %s RTSP/1.0\r\nCSeq:%d\r\nforceiframe:1\r\nUser-agent:%s\r\n\r\n";
		sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_UserAgent);
	}
	else
	{
		const char* format = "FORCEIFRAME %s RTSP/1.0\r\nCSeq:%d\r\nforceiframe:1\r\nUser-agent:%s\r\n\r\n";
		sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_UserAgent);
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GenerateOption() {
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}
	const char* format = "HEARTBEAT %s RTSP/1.0\r\nCSeq:%lu\r\nSession:%s\r\nUser-Agent:%s\r\n\r\n";
	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_sSession, m_UserAgent);

	return HPR_TRUE;
}


HPR_BOOL CRtspClientEngine::GeneratePause() {
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}
	const char* format = "PAUSE %s RTSP/1.0\r\nCSeq:%d\r\nSession:%s\r\nUser-Agent:%s\r\n\r\n";
	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_sSession, m_UserAgent);

	return HPR_TRUE;
}


HPR_BOOL CRtspClientEngine::GenerateResume()
{
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}

	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);

	const char* format = "PLAY %s RTSP/1.0\r\nCSeq:%d\r\nSession:%s\r\nUser-Agent:%s\r\n\r\n";
	sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_sSession, m_UserAgent);

	return HPR_TRUE;
}


HPR_BOOL CRtspClientEngine::GenerateHeartBeat() {
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}

	const char* format = "HEARTBEAT %s RTSP/1.0\r\nCSeq:%lu\r\nSession:%s\r\nUser-Agent:%s\r\n\r\n";
	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_sSession, m_UserAgent);

	return HPR_TRUE;
}


HPR_BOOL CRtspClientEngine::GenerateTearDown() {
	if (m_sSendBuf == NULL) {
		return HPR_FALSE;
	}
	const char* format = "TEARDOWN %s RTSP/1.0\r\nCSeq: %lu\r\nSession:%s\r\nUser-Agent: %s\r\n\r\n";
	memset(m_sSendBuf, 0, RTSP_RTSPBUF_SIZE);
	sprintf(m_sSendBuf, format, m_sRtspUrl, m_iSeq++, m_sSession, m_UserAgent);

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::SendRtspRequest()
{
	HPR_INT32 nPackLen = 0;
	struct timeval timeout = {10, 0};
	nPackLen = strlen((const char*)m_sSendBuf);
	LOGD("**************send cmd begin*******************\n");
	LOGD("msg:\t%s", m_sSendBuf);
	LOGD("**************send cmd end*******************\n");
	fd_set fdwrite;
	FD_ZERO(&fdwrite);
	FD_SET(m_iRTSPSocket, &fdwrite);

	if (HPR_Select(m_iRTSPSocket + 1, NULL, &fdwrite, NULL, &timeout) <= 0)
	{
        LOGD("CRtspClientEngine[%d]::SendRequest() HPR_Select()<=0, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
        return HPR_FALSE;
    }

//	if (HPR_FdIsSet(m_iRTSPSocket, &fdwrite) <= 0)
//	{
//        CRtspClientPrint::Print("CRtspClientEngine[%d]::SendRequest() HPR_FdIsSet()<=0, errorCode:%u and system errorCode:%u\n", \
//								m_iEngineerIndex, \
//								HPR_GetLastError(), \
//								HPR_GetSystemLastError());
//        return HPR_FALSE;
//    }

	if (HPR_Send(m_iRTSPSocket, m_sSendBuf, HPR_INT32(nPackLen)) != HPR_INT32(nPackLen))
	{
		LOGD("CRtspClientEngine[%d]::SendRequest() HPR_Send() fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
		return HPR_FALSE;
	}

	LOGD("CRtspClientEngine[%d]::SendRequest() HPR_Send() success, errorCode:%u and system errorCode:%u\n", \
							m_iEngineerIndex, \
							HPR_GetLastError(), \
							HPR_GetSystemLastError());

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::RecvRtspResponse()
{
	if (m_sRecvBuf == NULL)
	{
		LOGD("CRtspClientEngine::RecvRtspResponse() RecvBuf is NULL\n");
		return HPR_FALSE;
	}
	memset(m_sRecvBuf, 0, RTSP_RTSPBUF_SIZE);

	timeval timeout = {40, 0};
	fd_set oReadSet;
	HPR_INT32 iRet = 0;
	HPR_BOOL bRet = HPR_FALSE;
	int iRecvLen = 0;
	m_iRecvTotalLen = 0;
	m_bVTDUStackErr = HPR_FALSE;

	char magic = '\0';
	char channel = '\0';
	HPR_INT16 len = 0;

	while (true)
	{
		FD_ZERO(&oReadSet);
		FD_SET(m_iRTSPSocket, &oReadSet);
		iRet = HPR_Select(m_iRTSPSocket + 1, &oReadSet, NULL, NULL, &timeout);
		if (iRet < 0)
		{
			LOGD("CRtspClientEngine[%d]::RecvRtspResponse() HPR_Select()<0, errorCode:%u and system errorCode:%u\n", \
									m_iEngineerIndex, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());

			bRet = HPR_FALSE;
			break;
		}
		else if (iRet == 0)
		{
			LOGD("CRtspClientEngine[%d]::RecvRtspResponse() HPR_Select()=0, errorCode:%u and system errorCode:%u\n", \
									m_iEngineerIndex, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());

			bRet = HPR_TRUE;
			break;
		}

		iRecvLen = HPR_Recv(m_iRTSPSocket, m_sRecvBuf + m_iRecvTotalLen, RTSP_RTSPBUF_SIZE - m_iRecvTotalLen);
		if (iRecvLen <= 0)
		{
			LOGD("CRtspClientEngine::RecvRtspResponse() HPR_Recv() [%d]<= 0, errorCode:%d and system errorCode:%d\n", \
									iRecvLen, \
									HPR_GetLastError(), \
									HPR_GetSystemLastError());
			bRet = HPR_FALSE;
			break;
		}
		else
		{
			//LOGD("Len:%d, Content:%s", iRecvLen, m_sRecvBuf);
			m_iRecvTotalLen += iRecvLen;
			iRecvLen = 0;
		}
		LOGD("CRtspClientEngine[%d]::RecvRtspResponse() success, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());

		// 错误堆栈判断
		memcpy(&magic, m_sRecvBuf, 1);
		memcpy(&channel, m_sRecvBuf + 1, 1);
		memcpy(&len, m_sRecvBuf + 2, 2);
		if ((0x24 == magic) && (0x64 == channel))
		{
			LOGD("CRtspClientEngine::ParseErrStackPacket magic-channel:%d--%d \n", magic, channel);
			if (m_iRecvTotalLen < ntohs(len))
			{
				// 数据不全，继续收
				LOGD("CRtspClientEngine::totalLen[%d] < packageLen[%d]", m_iRecvTotalLen, ntohs(len));
				continue;
			}
			if (DealStreamErrInfo(m_iRecvTotalLen, ntohs(len)))
			{
				LOGD("CRtspClientEngine::ParseErrStackPacket is success \n");
				m_bVTDUStackErr = HPR_TRUE;
				bRet = HPR_FALSE;
				break;
			}
		}
		if (m_iRecvTotalLen < RTSP_RTSPBUF_SIZE)
		{
			m_sRecvBuf[m_iRecvTotalLen] = '\0';
		}

		bool bTmp = m_oRtspParser.IsEnd(m_sRecvBuf, m_iRecvTotalLen);
		if (bTmp)
		{
			bRet = HPR_TRUE;
			break;
		}

		LOGD("CRtspClientEngine::RecvRtspResponse() Not Recv Finished, continue to Read\n");
		continue;
	}

	LOGD("**************recv cmd begin*******************\n");
	LOGD("%s", m_sRecvBuf);
	LOGD("**************recv cmd end*******************\n");

	return bRet;
}

HPR_BOOL CRtspClientEngine::ParseDescribePacket()
{
	// RTSP 交互状态码判断
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		if (m_statusCode == RTSP_STATUS_302 || m_statusCode == RTSP_STATUS_301)
		{
			// 需要重定向
			LOGD("CRtspClientEngine::ParseDescribePacket() Status Code is 302, get redirect url\n");
		}
		else
		{
			LOGD("CRtspClientEngine::ParseDescribePacket() Status Code is:%d\n", m_statusCode);
			long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
			if (errCode == RTSP_STATUS_UNK)
			{
				g_nRtspClientErrorCode = RTSPCLIENT_DESCRIBE_STATUS_NO_200OK;
			}
			else
			{
				g_nRtspClientErrorCode = errCode;
			}
		}
		return HPR_FALSE;
	}

	char acceptptn[MAX_ACCEPTTYEP_LEN + 1];
	memset(acceptptn, 0, MAX_ACCEPTTYEP_LEN + 1);
	bool bRet = m_oRtspParser.GetValueIngoreCase(m_sRecvBuf, (char*)"Content-Type", acceptptn, MAX_ACCEPTTYEP_LEN + 1);
	if (!bRet)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_PARSE_DESCRIBE_FAIL;
		return HPR_FALSE;
	}

	if(HPR_Strncasecmp(acceptptn, "application/sdp", 15) != 0)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_PARSE_DESCRIBE_FAIL;
		return HPR_FALSE;
	}

	if(m_iVersion == VERSION_MAG)
	{
		HIK_MEDIAINFO mediaInfo;
		memset(&mediaInfo, 0, sizeof(HIK_MEDIAINFO));
		mediaInfo.media_fourcc = 0x484B4D49;
		// 海康264码率
		mediaInfo.system_format = 0x0004;
		mediaInfo.video_format = 0x0001;

		memcpy(m_oSdpInfo.header, &mediaInfo, 40);
		m_oSdpInfo.headlen = 40;

		m_oSdpInfo.mediacount = 1;
		memcpy(m_oSdpInfo.mediaurl[0], m_sRtspUrl, strlen(m_sRtspUrl));

	}
	else {
		bRet = m_oRtspParser.GetSdpUrl(m_sRecvBuf, m_iRecvTotalLen, &m_oSdpInfo, m_sRtspUrl);
		if (!bRet)
		{
			LOGD("CRtspClientEngine::ParseDescribePacket() GetSdpUrl Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_DESCRIBE_FAIL;
			return HPR_FALSE;
		}
	}

	PritfMediaHeader(m_oSdpInfo.header, m_oSdpInfo.headlen);

	return HPR_TRUE;
}

const char * CRtspClientEngine::fetchMediaHeader(int *len)
{
    *len = m_oSdpInfo.headlen;
    return m_oSdpInfo.header;
}

void CRtspClientEngine::PritfMediaHeader(const char * mediaheader, int len)
{
	LOGD("CRtspClientEngine::PritfMediaHeader()\n");
	LOGD(mediaheader);

	unsigned int media_fourcc;
	memcpy(&media_fourcc, mediaheader, 4);

	if(media_fourcc == HKMI) {
		HIK_MEDIAINFO hikMediaInfo;
		memset(&hikMediaInfo, 0, sizeof(HIK_MEDIAINFO));
		memcpy(&hikMediaInfo, mediaheader, sizeof(HIK_MEDIAINFO));

		LOGD("HKMI Media Information");

		if(hikMediaInfo.system_format == SYSTEM_NULL)
		{
			LOGD("system format: no ");
		}
		else if(hikMediaInfo.system_format == SYSTEM_HIK)
		{
			LOGD("system format: hik");
		}
		else if(hikMediaInfo.system_format == SYSTEM_MPEG2_PS)
		{
			LOGD("system format: PS");
		}
		else if(hikMediaInfo.system_format == SYSTEM_MPEG2_TS)
		{
			LOGD("system format: TS");
		}
		else if(hikMediaInfo.system_format == SYSTEM_RTP)
		{
			LOGD("system format: rtp");
		}
		else
		{
			LOGD("system format:%d", hikMediaInfo.system_format);
		}

		if(hikMediaInfo.video_format == VIDEO_HIK264)
		{
			LOGD("video format: h.264 ");
		}
		else if(hikMediaInfo.video_format == VIDEO_AVC264)
		{
			LOGD("video format: standard h.264");
		}
		else if(hikMediaInfo.video_format == VIDEO_MPEG2)
		{
			LOGD("video format: standard MPEG2");
		}
		else if(hikMediaInfo.video_format == VIDEO_MPEG4)
		{
			LOGD("video format: standard VIDEO_MPEG4");
		}
		else if(hikMediaInfo.video_format == VIDEO_MJPEG)
		{
			LOGD("video format: Motion Jpeg");
		}
		else if(hikMediaInfo.video_format == VIDEO_SVC264)
		{
			LOGD("video format: SVC");
		}
		else if(hikMediaInfo.video_format == VIDEO_WMV9)
		{
			LOGD("video format: WMV9");
		}
		else if(hikMediaInfo.video_format == VIDEO_VC1)
		{
			LOGD("video format: VC1");
		}
		else if(hikMediaInfo.video_format == VIDEO_REAL)
		{
			LOGD("video format: REAL");
		}
		else
		{
			LOGD("video format:%d", hikMediaInfo.video_format);
		}

		LOGD("CRtspClientEngine::RecogMediaHeader->audio format:%d", hikMediaInfo.audio_format);
		LOGD("CRtspClientEngine::RecogMediaHeader->audio channels:%d", hikMediaInfo.audio_channels);
		LOGD("CRtspClientEngine::RecogMediaHeader->audio: %d bits per sample", hikMediaInfo.audio_bits_per_sample);
		LOGD("CRtspClientEngine::RecogMediaHeader->audio samples rate:%d", hikMediaInfo.audio_samplesrate);
		LOGD("CRtspClientEngine::RecogMediaHeader->audio samples rate:%d", hikMediaInfo.audio_bitrate);
		LOGD("...............media information end..............");
	}
	else if(media_fourcc == HVAG){
		HVAG_MEDIA_HEAD hvagHediaInfo;
		memset(&hvagHediaInfo, 0, sizeof(HVAG_MEDIA_HEAD));
		memcpy(&hvagHediaInfo, mediaheader, sizeof(HVAG_MEDIA_HEAD));

		LOGD("HVAG Media Information");

		unsigned short system_format = ntohs(hvagHediaInfo.system_format);
		if(system_format == HVAG_SYSTEM_NULL)
		{
			LOGD("system format: no ");
		}
		else if(system_format == HVAG_SYSTEM_RAW)
		{
			LOGD("system format: 有帧边界原始封装格式");
		}
		else if(system_format == HVAG_SYSTEM_PS)
		{
			LOGD("system format: PS封装格式");
		}
		else if(system_format == HVAG_SYSTEM_RTP)
		{
			LOGD("system format: RTP封装格式");
		}
		else if(system_format == HVAG_SYSTEM_RTP_PS)
		{
			LOGD("system format: RTP+PS（国标）封装格式");
		}
		else if(system_format == HVAG_PRIVATE_NULL)
		{
			LOGD("system format: 增加私有头，无帧边界原始封装格式");
		}
		else if(system_format == HVAG_PRIVATE_RAW)
		{
			LOGD("system format: 增加私有头，有帧边界原始封装格式");
		}
		else if(system_format == HVAG_PRIVATE_PS)
		{
			LOGD("system format: 增加私有头，PS封装格式，实际不会使用");
		}
		else if(system_format == HVAG_PRIVATE_RTP)
		{
			LOGD("system format: 增加私有头，RTP封装格式");
		}
		else if(system_format == HVAG_PRIVATE_RTP_PS)
		{
			LOGD("system format: 增加私有头，RTP+PS（国标）封装格式");
		}
		else
		{
			LOGD("system format:%d", system_format);
		}

		unsigned short video_format  = ntohs(hvagHediaInfo.video_format);
		if(video_format == HVAG_VIDEO_RAW)
		{
			LOGD("video format: 原始码流编码 ");
		}
		else if(video_format == HVAG_VIDEO_H264)
		{
			LOGD("video format: H264编码");
		}
		else if(video_format == HVAG_VIDEO_MPEG4)
		{
			LOGD("video format: MPEG4编码");
		}
		else
		{
			LOGD("video format:%d", video_format);
		}

		LOGD("CRtspClientEngine::RecogMediaHeader->stream_head_len:%d", ntohs(hvagHediaInfo.stream_head_len));
	}
	else
	{
		LOGD("CRtspClientEngine::RecogMediaHeader not recogize, media_fourcc:%d", media_fourcc);
	}
}

HPR_BOOL CRtspClientEngine::ParseSetupPacket(int medieIndex)
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParseSetupPacket() Status Code is not 200\n");
		long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_SETUP_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
		return HPR_FALSE;
	}

	if (m_sSession != NULL)
	{
		memset(m_sSession, 0, RTSP_SESSION_SIZE);
		bool bRet = m_oRtspParser.GetSession(m_sRecvBuf, (char*)"Session", m_sSession, RTSP_SESSION_SIZE);
		if (!bRet)
		{
			LOGD("CRtspClientEngine::RecvRtspResponse() GetSession Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_SETUP_FAIL;
			return HPR_FALSE;
		}
	}

	if (m_sTransportBuf != NULL)
	{
		memset(m_sTransportBuf, 0, RTSP_TRANSPORT_SIZE);
		bool bRet = m_oRtspParser.GetValueIngoreCase(m_sRecvBuf, (char*)"Transport", m_sTransportBuf, RTSP_TRANSPORT_SIZE);
		if (!bRet)
		{
			LOGD("CRtspClientEngine::RecvRtspResponse() GetTransport Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_SETUP_FAIL;
			return HPR_FALSE;
		}
	}

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
		int iRtpServerPort = 0;
		bool bRet = m_oRtspParser.GetServerPort(m_sTransportBuf, (char*)"server_port", m_sServerPortBuf, RTSP_SERVERPORT_SIZE, iRtpServerPort);
		if (!bRet)
		{
			LOGD("CRtspClientEngine::RecvRtspResponse() GetServerPort Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_SETUP_FAIL;
			return HPR_FALSE;
		}

		m_oUdpTcpPortPair[medieIndex].SetRTPServerPort(iRtpServerPort);
		m_oUdpTcpPortPair[medieIndex].SetRTCPServerPort(iRtpServerPort + 1);
		m_oUdpTcpPortPair[medieIndex].SetRTPServerAddr(m_iRTSPServerAddr);

		char serverSsrc[32];
		memset(serverSsrc, 0, 32);
		bRet = m_oRtspParser.GetServerSsrc(m_sTransportBuf, (char*)"ssrc", serverSsrc, 32);
		if (!bRet)
		{
			//CRtspClientPrint::Print("CRtspClientEngine::RecvRtspResponse() GetServerSsrc Fail\n");
			m_iSSRC = 0;
		}
		else
		{
			sscanf(serverSsrc, "%X", &m_iSSRC);
		}
	}
	else if (m_iRtpProtocol == RTPRTSP_TRANSMODE)
	{
		if (m_sInterleavedBuf == NULL)
		{
			LOGD("CRtspClientEngine::ParseSetupPacket() m_sInterleavedBuf is NULL\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_SETUP_FAIL;
			return HPR_FALSE;
		}

		int iInterleaved = 0;
		memset(m_sInterleavedBuf, 0, RTSP_INTERLEAVE_SIZE);
		bool bRet = m_oRtspParser.GetInterleaved(m_sTransportBuf, (char*)"interleaved", m_sInterleavedBuf, RTSP_INTERLEAVE_SIZE, iInterleaved);
		if (!bRet)
		{
			LOGD("CRtspClientEngine::RecvRtspResponse() GetInterleaved Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_PARSE_SETUP_FAIL;
			return HPR_FALSE;
		}
	}
	else
	{

	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParsePlayPacket()
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParsePlayPacket() Status Code is not 200\n");
		long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_PLAY_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseRandomPlayPacket()
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParseRandomPlayPacket() Status Code is not 200\n");
		long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RANDOMPLAY_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseChanageRatePacket()
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParseChanageRatePacket() Status Code is not 200\n");
		long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_CHANGERATE_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParsePausePacket()
{
    // 解析状态码
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
    if (m_statusCode != RTSP_STATUS_200)
    {
        LOGD("CRtspClientEngine::ParsePausePacket() Status Code is not 200\n");
		long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_PAUSE_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
        return HPR_FALSE;
    }
    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseResumePacket()
{
    // 解析状态码
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
    if (m_statusCode != RTSP_STATUS_200)
    {
        LOGD("CRtspClientEngine::ParseResumePacket() Status Code is not 200\n");
        long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_RESUME_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
        return HPR_FALSE;
    }
    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseForceIFramePacket()
{
    // 解析状态码
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
    if (m_statusCode != RTSP_STATUS_200)
    {
        LOGD("CRtspClientEngine::ParseForceIFramePacket() Status Code is not 200\n");
        long errCode = (long)m_oRtspParser.GetExtErrCode(m_sRecvBuf);
		if (errCode == RTSP_STATUS_UNK)
		{
			g_nRtspClientErrorCode = RTSPCLIENT_FORCEIFRAME_STATUS_NO_200OK;
		}
		else
		{
			g_nRtspClientErrorCode = errCode;
		}
        return HPR_FALSE;
    }
    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseOptionsPacket()
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParseOptionsPacket() Status Code is not 200\n");
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseTearDownPacket()
{
	m_statusCode = m_oRtspParser.GetStatusCodeIndex(m_sRecvBuf);
	if (m_statusCode != RTSP_STATUS_200)
	{
		LOGD("CRtspClientEngine::ParseOptionsPacket() Status Code is not 200\n");
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::ParseErrStackPacket()
{
	// 错误堆栈判断
	char magic = '\0';
	char channel = '\0';
	HPR_INT16 len = 0;

	memcpy(&magic, m_sRecvBuf, 1);
	memcpy(&channel, m_sRecvBuf + 1, 1);
	memcpy(&len, m_sRecvBuf + 2, 2);

	LOGD("CRtspClientEngine::ParseErrStackPacket magic:%d--%d \n", magic, channel);
	if ((0x24 == magic) && (0x64 == channel))
	{
	    if (m_iRecvTotalLen < ntohs(len))
	    {
	        // 码流不全，继续收
	    	LOGD("CRtspClientEngine::DealStreamErrInfo() totalLen[%d] < packageLen[%d]", m_iRecvTotalLen, ntohs(len));
	        return HPR_FALSE;
	    }
		if (DealStreamErrInfo(m_iRecvTotalLen, ntohs(len)))
		{
			LOGD("CRtspClientEngine::ParseErrStackPacket DealStreamErrInfo is success \n");
			return HPR_TRUE;
		}
	}

	LOGD("CRtspClientEngine::ParseErrStackPacket no stack error \n");
	return HPR_FALSE;
}

HPR_BOOL CRtspClientEngine::ConvertToStdTime(pABS_TIME pTime, string & stdTime)
{
    if (NULL == pTime)
    {
        return HPR_FALSE;
    }

    string year, month, day, hour, min, sec;
    char str_time[5] = {0};
    sprintf(str_time, "%u", pTime->dwYear);
    year = str_time;
    if(pTime->dwMonth < 10)
    {
        sprintf(str_time, "0%u", pTime->dwMonth);
    }
    else
    {
        sprintf(str_time, "%u", pTime->dwMonth);
    }
    month = str_time;
    //CRtspClientPrint::Print("month:%s", const_cast<char*>(month.c_str()));


    if(pTime->dwDay < 10)
    {
        sprintf(str_time, "0%u", pTime->dwDay);
    }
    else
    {
        sprintf(str_time, "%u", pTime->dwDay);
    }
    day = str_time;

    if(pTime->dwHour < 10)
    {
        sprintf(str_time, "0%u", pTime->dwHour);
    }
    else
    {
        sprintf(str_time, "%u", pTime->dwHour);
    }
    hour = str_time;

    if(pTime->dwMinute < 10)
    {
        sprintf(str_time, "0%u", pTime->dwMinute);
    }
    else
    {
        sprintf(str_time, "%u", pTime->dwMinute);
    }
    min = str_time;

    if(pTime->dwSecond < 10)
    {
        sprintf(str_time, "0%u", pTime->dwSecond);
    }
    else
    {
        sprintf(str_time, "%u", pTime->dwSecond);
    }
    sec = str_time;

    stdTime = year + month + day + "T" + hour + min + sec + "Z";

    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::GetRedirectUrl()
{
	LOGD("CRtspClientEngine::Redirect()\n");

	m_statusCode = RTSP_STATUS_UNK;

	memset(m_sRtspUrl, 0, MAX_URL_LEN);
	bool ret = m_oRtspParser.GetRedirectUrl(m_sRecvBuf, m_sRtspUrl);
	if (!ret)
	{
		g_nRtspClientErrorCode = RTSPCLIENT_DECRIBE_REDIRCT_FAIL;
		return HPR_FALSE;
	}

	 return HPR_TRUE;
}

HPR_VOIDPTR CRtspClientEngine::OptionsProc(HPR_VOIDPTR pParam)
{
	CRtspClientEngine* pPtr = (CRtspClientEngine*)pParam;
	if (pPtr == NULL)
	{
		return NULL;
	}

	// 每5s发送一个心跳
	int iCount = 10;
	for (;;)
	{
		if (pPtr->m_bStopOptions)
		{
			LOGD("CRtspClientEngine::OptionsProc() stop\n");
			break;
		}

		if (iCount >= 10)
		{
			if (pPtr->Options())
			{
				iCount = 0;
			}
		}
		else
		{
			HPR_Sleep(500);
			iCount++;
		}
	}

	return NULL;
}

HPR_BOOL CRtspClientEngine::DisConnectServer()
{
    // 关闭socket
    if (m_iRTSPSocket == HPR_INVALID_SOCKET)
    {
    	LOGD("CRtspClientEngine::DisConnectServer m_iRTSPSocket HPR_INVALID_SOCKET\n");
    	return HPR_FALSE;
    }

    HPR_INT32 iRet = HPR_ShutDown(m_iRTSPSocket, HPR_SHUT_RDWR);
    if(iRet != HPR_OK)
    {
    	LOGD("CRtspClientEngine::DisConnectServer HPR_ShutDown fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
    }
    iRet = HPR_CloseSocket(m_iRTSPSocket);
	if(iRet != HPR_OK)
	{
		LOGD("CRtspClientEngine::DisConnectServer HPR_ShutDown fail, errorCode:%u and system errorCode:%u\n", \
								m_iEngineerIndex, \
								HPR_GetLastError(), \
								HPR_GetSystemLastError());
	}
	m_iRTSPSocket = HPR_INVALID_SOCKET;

    LOGD("CRtspClientEngine::DisConnectServer success\n");
    return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::Options()
{
	LOGD("CRtspClientEngine::Options()\n");

	if (m_sSendBuf == NULL)
	{
		LOGD("CRtspClientEngine::Options() SendBuf is NULL\n");
		return HPR_FALSE;
	}
	HPR_BOOL bRet = GenerateOption();
	if (!bRet)
	{
		return HPR_FALSE;
	}

    bRet = SendRtspRequest();
	if (!bRet)
	{
		return HPR_FALSE;
	}

	// add by hyf
    //RecvRtspResponse();

	if (m_iRtpProtocol == RTPUDP_TRANSMODE || m_iRtpProtocol == RTPTCP_TRANSMODE)
	{
        bRet = RecvRtspResponse();
        if (!bRet)
        {
            return HPR_FALSE;
        }

        bRet = ParseOptionsPacket();
		if (!bRet)
		{
			return HPR_FALSE;
		}
	}

    return HPR_TRUE;
}

void CRtspClientEngine::StartOptionsThread()
{
	m_bStopOptions = HPR_FALSE;

	m_hOptionsThread = HPR_Thread_Create(OptionsProc, this, 0);
	if (m_hOptionsThread == HPR_INVALID_HANDLE)
	{
		LOGD("CRtspClientEngine::StartKeepAliveThread create keep alive thread fail\n");
	}
}

void CRtspClientEngine::StopOptionsThread()
{
	m_bStopOptions = HPR_TRUE;

	if (m_hOptionsThread != HPR_INVALID_HANDLE)
	{
		HPR_Thread_Wait(m_hOptionsThread);
		LOGD("CRtspClientEngine::StopOptionsThread() wait end");
		m_hOptionsThread = HPR_INVALID_HANDLE;
	}
}

HPR_BOOL CRtspClientEngine::StartRtpTcpTrans()
{
	LOGD("CRtspClientEngine::StartRtpTcpTrans()\n");
	if (m_pRtpTcpTransEngine == NULL)
	{
		m_pRtpTcpTransEngine = new CRtpTcpTransEngine();
		if (m_pRtpTcpTransEngine == NULL)
		{
			LOGD("CRtspClientEngine::StartRtpTcpTrans() new m_pRtpTcpTransEngine Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_MOLLOC_RTPTCPENGINE_FAIL;
			return HPR_FALSE;
		}
	}

	HPR_BOOL bRet = m_pRtpTcpTransEngine->Init(m_iEngineerIndex, m_pDataCallback, &m_oUdpTcpPortPair[0], this);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartRtpTcpTrans() Init m_pRtpTcpTransEngine Fail\n");

		delete m_pRtpTcpTransEngine;
		m_pRtpTcpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_INIT_RTPTCPENGINE_FAIL;
		return HPR_FALSE;
	}

	bRet = m_pRtpTcpTransEngine->StartRtpTrans();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartRtpTcpTrans() Start RTPTransEngine Fail\n");

		m_pRtpTcpTransEngine->Release();
		delete m_pRtpTcpTransEngine;
		m_pRtpTcpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_START_RTPTCPENGINE_FAIL;
		return HPR_FALSE;
	}

	LOGD("CRtspClientEngine::StartRtpTcpTrans() Success\n");
	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::StopRtpTcpTrans()
{
	/*if (m_pRtcpTransEngine != NULL)
	{
		m_pRtcpTransEngine->StopRtcpTrans();
		m_pRtcpTransEngine->Release();

		delete m_pRtcpTransEngine;
		m_pRtcpTransEngine = NULL;
	}*/

	if (m_pRtpTcpTransEngine != NULL)
	{
		m_pRtpTcpTransEngine->StopRtpTrans();
		m_pRtpTcpTransEngine->Release();

		delete m_pRtpTcpTransEngine;
		m_pRtpTcpTransEngine = NULL;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::StartRtpUdpTrans()
{
	if (m_pRtpUdpTransEngine == NULL)
	{
		m_pRtpUdpTransEngine = new CRtpUdpTransEngine();
		if (m_pRtpUdpTransEngine == NULL)
		{
			LOGD("CRtspClientEngine::StartUDPTrans() new m_pRtpUdpTransEngine Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_MOLLOC_RTPUDPENGINE_FAIL;
			return HPR_FALSE;
		}
	}

	unsigned int iClientSsrc = 0;

	HPR_BOOL bRet = m_pRtpUdpTransEngine->Init(m_iEngineerIndex, m_pDataCallback, iClientSsrc, &m_oUdpTcpPortPair[0], this);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartUDPTrans() Init m_pRtpUdpTransEngine Fail\n");

		delete m_pRtpUdpTransEngine;
		m_pRtpUdpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_INIT_RTPUDPENGINE_FAIL;
		return HPR_FALSE;
	}

	bRet = m_pRtpUdpTransEngine->StartRtpTrans();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartUDPTrans() Start m_pRtpUdpTransEngine Fail\n");

		m_pRtpUdpTransEngine->Release();
		delete m_pRtpUdpTransEngine;
		m_pRtpUdpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_START_RTPUDPENGINE_FAIL;
		return HPR_FALSE;
	}

	if (m_pRtcpUdpTransEngine == NULL)
	{
		m_pRtcpUdpTransEngine = new CRtcpUdpTransEngine();
		if (m_pRtcpUdpTransEngine == NULL)
		{
			LOGD("CRtspClientEngine::StartUDPTrans() new RTCPTransEngine Fail\n");

			m_pRtpUdpTransEngine->StopRtpTrans();
			m_pRtpUdpTransEngine->Release();
			delete m_pRtpUdpTransEngine;
			m_pRtpUdpTransEngine = NULL;
			g_nRtspClientErrorCode = RTSPCLIENT_MOLLOC_RTCPUDPENGINE_FAIL;
			return HPR_FALSE;
		}
	}

	bRet = m_pRtcpUdpTransEngine->Init(m_iEngineerIndex, m_pRtpUdpTransEngine, &m_oUdpTcpPortPair[0], this);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartUDPTrans() Init m_pRtcpUdpTransEngine Fail\n");

		m_pRtpUdpTransEngine->StopRtpTrans();
		m_pRtpUdpTransEngine->Release();
		delete m_pRtpUdpTransEngine;
		m_pRtpUdpTransEngine = NULL;

		delete m_pRtcpUdpTransEngine;
		m_pRtcpUdpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_INIT_RTCPUDPENGINE_FAIL;
		return HPR_FALSE;
	}

	bRet = m_pRtcpUdpTransEngine->StartRtcpTrans();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartUDPTrans() Start RTCPTransEngine Fail\n");

		m_pRtpUdpTransEngine->StopRtpTrans();
		m_pRtpUdpTransEngine->Release();
		delete m_pRtpUdpTransEngine;
		m_pRtpUdpTransEngine = NULL;

		m_pRtcpUdpTransEngine->Release();
		delete m_pRtcpUdpTransEngine;
		m_pRtcpUdpTransEngine = NULL;
		g_nRtspClientErrorCode = RTSPCLIENT_START_RTCPUDPENGINE_FAIL;
		return HPR_FALSE;
	}

	return HPR_TRUE;
}
HPR_BOOL CRtspClientEngine::StopRtpUdpTrans()
{
	if (m_pRtcpUdpTransEngine != NULL)
	{
		m_pRtcpUdpTransEngine->StopRtcpTrans();
		m_pRtcpUdpTransEngine->Release();

		delete m_pRtcpUdpTransEngine;
		m_pRtcpUdpTransEngine = NULL;
	}

	if (m_pRtpUdpTransEngine != NULL)
	{
		m_pRtpUdpTransEngine->StopRtpTrans();
		m_pRtpUdpTransEngine->Release();

		delete m_pRtpUdpTransEngine;
		m_pRtpUdpTransEngine = NULL;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::StartRtpRtspTrans()
{
	if (m_pRtpRtcpTransEngine == NULL)
	{
		m_pRtpRtcpTransEngine = new CRtpRtcpTransEngine();
		if (m_pRtpRtcpTransEngine == NULL)
		{
			LOGD("CRtspClientEngine::StartTCPTrans() new CRtpRtcpTransEngine Fail\n");
			g_nRtspClientErrorCode = RTSPCLIENT_MOLLOC_RTPRTSPENGINE_FAIL;
			return HPR_FALSE;
		}
	}

	HPR_BOOL bRet = m_pRtpRtcpTransEngine->Init(m_iEngineerIndex, m_pDataCallback, this);
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartTCPTrans() CRtpRtcpTransEngine Init Fail\n");
		if (m_pRtpRtcpTransEngine != NULL)
		{
			delete m_pRtpRtcpTransEngine;
			m_pRtpRtcpTransEngine = NULL;
		}

		g_nRtspClientErrorCode = RTSPCLIENT_INIT_RTPRTSPENGINE_FAIL;
		return HPR_FALSE;
	}

	bRet = m_pRtpRtcpTransEngine->StartRtpRtcpTrans();
	if (!bRet)
	{
		LOGD("CRtspClientEngine::StartTCPTrans() StartRtpRtcpTrans Fail\n");
		if (m_pRtpRtcpTransEngine != NULL)
		{
			m_pRtpRtcpTransEngine->Release();
			delete m_pRtpRtcpTransEngine;
			m_pRtpRtcpTransEngine = NULL;
		}

		g_nRtspClientErrorCode = RTSPCLIENT_START_RTPRTSPENGINE_FAIL;
		return HPR_FALSE;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::StopRtpRtspTrans()
{
	if (m_pRtpRtcpTransEngine != NULL)
	{
		m_pRtpRtcpTransEngine->StopRtpRtcpTrans();
		m_pRtpRtcpTransEngine->Release();

		delete m_pRtpRtcpTransEngine;
		m_pRtpRtcpTransEngine = NULL;
	}

	return HPR_TRUE;
}

HPR_BOOL CRtspClientEngine::DealStreamErrInfo(unsigned short totalLen, unsigned short packageLen)
{
    if (0 == packageLen)
    {
    	LOGD("CRtspClientEngine::DealStreamErrInfo() packageLen is 0");
        return HPR_FALSE;
    }

	int count = 0;
	ERRORSTACKINFO1 errStaInfo;
	memset(&errStaInfo, 0, sizeof(ERRORSTACKINFO1));

    int* netPrivateType = (int*)(m_sRecvBuf + 8);
    unsigned int privateType = ntohl(*netPrivateType);

    if (0x01 == privateType )
    {
		// 解析错误码结构体
    	char errInfo[MAX_URL_LEN] = {0};
        memset(errInfo, 0, MAX_URL_LEN);
        if (GetErrorStackInfoFromBuf(errInfo, MAX_URL_LEN, m_sRecvBuf + 12, packageLen))
        {
            memset(&errStaInfo, 0, sizeof(errStaInfo));
            errStaInfo.count = (int*)errInfo;
            errStaInfo.perrorInfo = (PERRORINFO1)(errInfo + 4);

            int iErr = errStaInfo.perrorInfo->errorCode + STREAM_CLIENT_ERRNO_START;
            LOGD("CRtspClientEngine::DealStreamErrInfo() ErrCode:[%d],ErrDes:%s\n", iErr, errStaInfo.perrorInfo->errorMsg);
            g_nRtspClientErrorCode = iErr;
            return HPR_TRUE;
        }
    }
    LOGD("CRtspClientEngine::DealStreamErrInfo() No err");
    return HPR_FALSE;
}

HPR_BOOL CRtspClientEngine::GetErrorStackInfoFromBuf(char* errbuf, int errbuflen, const char* buf, int buflen)
{
    if (buf == NULL || buflen < 4 || errbuf == NULL)
    {
    	LOGD("CRtspClientEngine::GetErrorStackInfoFromBuf() param is null");
        return HPR_FALSE;
    }

    memset(errbuf, 0, errbuflen);
    ERRORSTACKINFO1 error = {0};
    ERRORSTACKINFO1 after = {0};
    error.count = (int*)buf;
    after.count = (int*)errbuf;

    int count = (int)ntohl(*(error.count));
    *(after.count) = count;
    int parselen = (int)(sizeof(ERRORINFO1) * count + 4);

    if (count == 0 || buflen < parselen || errbuflen < parselen)
    {
    	LOGD("CRtspClientEngine::GetErrorStackInfoFromBuf() count == 0 || buflen < parselen || errbuflen < parselen");
        return HPR_FALSE;
    }

    error.perrorInfo = (PERRORINFO1)(buf + 4);
    after.perrorInfo = (PERRORINFO1)(errbuf + 4);
    for (int i = 0; i < count; i++)
    {
        memcpy((after.perrorInfo + i * sizeof(ERRORINFO1)), (error.perrorInfo + i * sizeof(ERRORINFO1)), sizeof(ERRORINFO1));
    }
    return HPR_TRUE;
}


/***************************************************************************/

