/*
 * CopyRight:	HangZhou Hikvision System Technology Co., Ltd. All	Right Reserved.
 * FileName:	hpr2_mutex.cpp
 * Desc:		hpr2 mutex.
 * Author:		zhaoyiji
 * Date:		2012-11-12
 * Contact:		zhaoyiji@hikvision.com.cn
 * History:		Created By Zhaoyiji 2012-11-12 (V0.1)
 * */

#include "hpr2_types.h"
#include "hpr2_mutex.h"

#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#else
#include <pthread.h>
#include <stdlib.h>
#endif

typedef struct hpr2_mutex_t
{
#if defined(_WIN32) || defined(_WIN64)
	CRITICAL_SECTION mutex;
#else
	pthread_mutex_t mutex;
#endif
}hpr2_mutex_t;

int hpr2_mutex_create(hpr2_mutex_t** handle, int flag)
{
	if (NULL == handle)
	{
		return HPR2_ERR;
	}

	*handle = (hpr2_mutex_t*)malloc(sizeof(hpr2_mutex_t));
	if (NULL == *handle)
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	(void)(flag);
	__try
	{
		InitializeCriticalSection(&(*handle)->mutex);
	}
	__except(GetExceptionCode() == STATUS_NO_MEMORY)
	{
		free(*handle);
		*handle = NULL;

		return HPR2_ERR;
	}

	return HPR2_OK;
#else
	int ret = 0;
	pthread_mutexattr_t attr;
	pthread_mutexattr_init(&attr);

	pthread_mutexattr_settype(&attr, flag);

	ret = pthread_mutex_init(&((*handle)->mutex), &attr); 

	pthread_mutexattr_destroy(&attr);
	
	if (ret != 0)
	{
		free(*handle);
		*handle = NULL;
	}

	return (ret == 0) ? HPR2_OK : HPR2_ERR;
#endif
}

int hpr2_mutex_lock(hpr2_mutex_t* handle)
{
	if (!handle)
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	EnterCriticalSection(&(handle->mutex));
	return HPR2_OK;
#else
	return pthread_mutex_lock(&(handle->mutex)) == 0 ? HPR2_OK : HPR2_ERR;
#endif
}

int hpr2_mutex_trylock(hpr2_mutex_t* handle)
{
	if ( (!handle))
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	TryEnterCriticalSection(&handle->mutex);
	return HPR2_OK;
#else
	return pthread_mutex_trylock(&(handle->mutex)) == 0 ? HPR2_OK : HPR2_ERR;
#endif
}

int hpr2_mutex_unlock(hpr2_mutex_t* handle)
{
	if ( (!handle))
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	LeaveCriticalSection(&handle->mutex);
	return HPR2_OK;
#else

	return pthread_mutex_unlock(&(handle->mutex)) == 0 ? HPR2_OK : HPR2_ERR;

#endif
}

int hpr2_mutex_destroy(hpr2_mutex_t* handle)
{
	if (!handle)
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	DeleteCriticalSection(&(handle->mutex));
#else
	 pthread_mutex_destroy(&(handle->mutex));
#endif

	 free(handle);
	 return HPR2_OK;
}
