/*
 * CopyRight:	HangZhou Hikvision System Technology Co., Ltd. All	Right Reserved.
 * FileName:	hpr2_rmm.cpp
 * Desc:		hpr2 reallocate memory(memory pool).
 * Author:		zhaoyiji
 * Date:		2012-11-12
 * Contact:		zhaoyiji@hikvision.com.cn
 * History:		Created By Zhaoyiji 2012-11-12 (V0.1)
 * */

#include "hpr2_types.h"
#include "hpr2_rmm.h"
#include "hpr2_mutex.h"
#include "MCRPrint.h"

#if defined (__linux__) || (__APPLE__)
#include <stdlib.h>
#include <string.h>
#endif

/*
 * fprintf
#include <stdio.h>
 */

#define HPR2_ALIGN(size, boundary) \
	(((size) + ((boundary) - 1)) & ~((boundary) - 1))

#define HPR2_ALIGN_DEFAULT(size) HPR2_ALIGN(size, 8)

typedef struct rmm_block_t
{
	hpr2_size_t size;
	hpr2_rmm_off_t prev;
	hpr2_rmm_off_t next;
}rmm_block_t;

typedef struct rmm_hdr_block_t
{
	hpr2_size_t abssize;
	hpr2_rmm_off_t firstused;
	hpr2_rmm_off_t firstfree;
}rmm_hdr_block_t;

#define RMM_HDR_BLOCK_SIZE (HPR2_ALIGN_DEFAULT(sizeof(rmm_hdr_block_t)))
#define RMM_BLOCK_SIZE (HPR2_ALIGN_DEFAULT(sizeof(rmm_block_t)))

typedef struct hpr2_rmm_t
{
	rmm_hdr_block_t* base;
	hpr2_size_t size;
	hpr2_mutex_t* lock;
}hpr2_rmm_t;

static int hpr2_rmm_lock(hpr2_mutex_t* lck)
{
	if (!lck)
	{
		return HPR2_OK;
	}

	return hpr2_mutex_lock(lck);
}

static int hpr2_rmm_unlock(hpr2_mutex_t* lck)
{
	if (!lck)
	{
		return HPR2_OK;
	}

	return hpr2_mutex_unlock(lck);
}

static hpr2_rmm_off_t find_block_by_offset(hpr2_rmm_t *rmm, hpr2_rmm_off_t next, 
										  hpr2_rmm_off_t find, int includes)
{
	hpr2_rmm_off_t prev = 0;

	while (next) {
		struct rmm_block_t *blk = (rmm_block_t*)((char*)rmm->base + next);

		if (find == next)
			return next;

		/* Overshot? */
		if (find < next)
			return includes ? prev : 0;

		prev = next;
		next = blk->next;
	}
	return includes ? prev : 0;
}

static hpr2_rmm_off_t find_block_of_size(hpr2_rmm_t *rmm, hpr2_size_t size)
{
	hpr2_rmm_off_t next = rmm->base->firstfree;
	hpr2_rmm_off_t best = 0;
	hpr2_rmm_off_t bestsize = 0;

	while (next) {
		struct rmm_block_t *blk = (rmm_block_t*)((char*)rmm->base + next);

		if (blk->size == size)
			return next;

		if (blk->size >= size) {
			/* XXX: sub optimal algorithm 
			* We need the most thorough best-fit logic, since we can
			* never grow our rmm, we are SOL when we hit the wall.
			*/
			if (!bestsize || (blk->size < bestsize)) {
				bestsize = blk->size;
				best = next;
			}
		}

		next = blk->next;
	}

	if (bestsize > RMM_BLOCK_SIZE + size) {
		struct rmm_block_t *blk = (rmm_block_t*)((char*)rmm->base + best);
		struct rmm_block_t *new_blk = (rmm_block_t*)((char*)rmm->base + best + size);

		new_blk->size = blk->size - size;
		new_blk->next = blk->next;
		new_blk->prev = best;

		blk->size = size;
		blk->next = best + size;

		if (new_blk->next) {
			blk = (rmm_block_t*)((char*)rmm->base + new_blk->next);
			blk->prev = best + size;
		}
	}

	return best;
}

static void move_block(hpr2_rmm_t *rmm, hpr2_rmm_off_t curr, int free)
{   
	struct rmm_block_t *blk = (rmm_block_t*)((char*)rmm->base + curr);

	/* close the gap */
	if (blk->prev) {
		struct rmm_block_t *prev = (rmm_block_t*)((char*)rmm->base + blk->prev);
		prev->next = blk->next;
	}
	else {
		if (free) {
			rmm->base->firstused = blk->next;
		}
		else {
			rmm->base->firstfree = blk->next;
		}
	}
	if (blk->next) {
		struct rmm_block_t *next = (rmm_block_t*)((char*)rmm->base + blk->next);
		next->prev = blk->prev;
	}

	/* now find it in the other list, pushing it to the head if required */
	if (free) {
		blk->prev = find_block_by_offset(rmm, rmm->base->firstfree, curr, 1);
		if (!blk->prev) {
			blk->next = rmm->base->firstfree;
			rmm->base->firstfree = curr;
		}
	}
	else {
		blk->prev = find_block_by_offset(rmm, rmm->base->firstused, curr, 1);
		if (!blk->prev) {
			blk->next = rmm->base->firstused;
			rmm->base->firstused = curr;
		}
	}

	/* and open it up */
	if (blk->prev) {
		struct rmm_block_t *prev = (rmm_block_t*)((char*)rmm->base + blk->prev);
		if (free && (blk->prev + prev->size == curr)) {
			/* Collapse us into our predecessor */
			prev->size += blk->size;
			curr = blk->prev;
			blk = prev;
		}
		else {
			blk->next = prev->next;
			prev->next = curr;
		}
	}

	if (blk->next) {
		struct rmm_block_t *next = (rmm_block_t*)((char*)rmm->base + blk->next);
		if (free && (curr + blk->size == blk->next)) {
			/* Collapse us into our successor */
			blk->size += next->size;
			blk->next = next->next;
			if (blk->next) {
				next = (rmm_block_t*)((char*)rmm->base + blk->next);
				next->prev = curr;
			}
		}
		else {
			next->prev = curr;
		}
	}
}

int hpr2_rmm_init(hpr2_rmm_t **rmm, void *lock, void *base, hpr2_size_t size)
{
	int rv;
	rmm_block_t *blk;

	/*
	fprintf(stderr, "RMM_HDR_BLOCK_SIZE = %ld\n", RMM_HDR_BLOCK_SIZE);
	fprintf(stderr, "RMM_BLOCK_SIZE = %ld\n", RMM_BLOCK_SIZE);
	*/

	if ((rv = hpr2_rmm_lock((hpr2_mutex_t*)lock)) != HPR2_OK)
	{
		return rv;
	}

	(*rmm) = (hpr2_rmm_t *)malloc(sizeof(hpr2_rmm_t));
	memset(*rmm, 0, sizeof(hpr2_rmm_t));
	(*rmm)->base = (rmm_hdr_block_t*)base;
	(*rmm)->size = size;
	(*rmm)->lock = (hpr2_mutex_t*)lock;

	(*rmm)->base->abssize = size;
	(*rmm)->base->firstused = 0;
	(*rmm)->base->firstfree = RMM_HDR_BLOCK_SIZE;

	blk = (rmm_block_t *)((char*)base + (*rmm)->base->firstfree);

	blk->size = size - (*rmm)->base->firstfree;
	blk->prev = 0;
	blk->next = 0;

	return hpr2_rmm_unlock((hpr2_mutex_t*)lock);
}

int hpr2_rmm_destroy(hpr2_rmm_t *rmm)
{
	int rv;
	rmm_block_t *blk;

	if ((rv = hpr2_rmm_lock(rmm->lock)) != HPR2_OK) {
		return rv;
	}
	/* Blast it all --- no going back :) */
	if (rmm->base->firstused) {
		hpr2_rmm_off_t curr = rmm->base->firstused;
		do {
			blk = (rmm_block_t *)((char*)rmm->base + curr);
			curr = blk->next;
			blk->next = blk->prev = 0;
		} while (curr);
		rmm->base->firstused = 0;
	}
	if (rmm->base->firstfree) {
		hpr2_rmm_off_t curr = rmm->base->firstfree;
		do {
			blk = (rmm_block_t *)((char*)rmm->base + curr);
			curr = blk->next;
			blk->next = blk->prev = 0;
		} while (curr);
		rmm->base->firstfree = 0;
	}
	rmm->base->abssize = 0;
	rmm->size = 0;

	rv = hpr2_rmm_unlock(rmm->lock);
	free(rmm);

	return rv;
}

hpr2_rmm_off_t hpr2_rmm_malloc(hpr2_rmm_t *rmm, hpr2_size_t reqsize)
{
	hpr2_size_t size;
	hpr2_rmm_off_t curr;

	size = HPR2_ALIGN_DEFAULT(reqsize) + RMM_BLOCK_SIZE;
	if (size < reqsize) {
//		LOGD("hpr2_rmm >> size < reqsize");
		return 0;
	}

	hpr2_rmm_lock(rmm->lock);

	curr = find_block_of_size(rmm, size);

	if (curr) {
		move_block(rmm, curr, 0);
		curr += RMM_BLOCK_SIZE;
	}

	hpr2_rmm_unlock(rmm->lock);
	return curr;
}

int hpr2_rmm_free(hpr2_rmm_t *rmm, hpr2_rmm_off_t curr)
{
	int rv;
	struct rmm_block_t *blk;

	/* A little sanity check is always healthy, especially here.
	* If we really cared, we could make curr compile-time
	*/
	if (curr < RMM_HDR_BLOCK_SIZE + RMM_BLOCK_SIZE) {
		return HPR2_ERR;
	}

	curr -= RMM_BLOCK_SIZE;

	blk = (rmm_block_t*)((char*)rmm->base + curr);

	if ((rv = hpr2_rmm_lock(rmm->lock)) != HPR2_OK) {
		return rv;
	}
	if (blk->prev) {
		struct rmm_block_t *prev = (rmm_block_t*)((char*)rmm->base + blk->prev);
		if (prev->next != curr) {
			hpr2_rmm_unlock(rmm->lock);
			return HPR2_ERR;
		}
	}
	else {
		if (rmm->base->firstused != curr) {
			hpr2_rmm_unlock(rmm->lock);
			return HPR2_ERR;
		}
	}

	if (blk->next) {
		struct rmm_block_t *next = (rmm_block_t*)((char*)rmm->base + blk->next);
		if (next->prev != curr) {
			hpr2_rmm_unlock(rmm->lock);
			return HPR2_ERR;
		}
	}

	/* Ok, it remained [apparently] sane, so unlink it
	*/
	move_block(rmm, curr, 1);

	return hpr2_rmm_unlock(rmm->lock);
}

void * hpr2_rmm_addr_get(hpr2_rmm_t *rmm, hpr2_rmm_off_t entity) 
{
	/* debug-sanity checking here would be good
	*/
	return (void*)((char*)rmm->base + entity);
}

hpr2_rmm_off_t hpr2_rmm_offset_get(hpr2_rmm_t *rmm, void* entity)
{
	/* debug, or always, sanity checking here would be good
	* since the primitive is hpr2_rmm_off_t, I don't mind penalizing
	* inverse conversions for safety, unless someone can prove that
	* there is no choice in some cases.
	*/
	return (hpr2_rmm_off_t)((char*)entity - (char*)rmm->base);
}

