/*
 * CopyRight:	HangZhou Hikvision System Technology Co., Ltd. All	Right Reserved.
 * FileName:	hpr2_thread.cpp
 * Desc:		hpr2 thread.
 * Author:		zhaoyiji
 * Date:		2012-11-12
 * Contact:		zhaoyiji@hikvision.com.cn
 * History:		Created By Zhaoyiji 2012-11-12 (V0.1)
 * */

#include "hpr2_thread.h"

#if defined(_WIN32) || defined(_WIN64)
#include <process.h>
#include <windows.h>
#else
#include <pthread.h>
#include <stdlib.h>
#endif

#define HPR2_ERR (-1)
#define HPR2_OK (0)

#define HPR2_THREAD_DEFAULT_STACKSIZE 1048576 //1MB

typedef struct hpr2_thread_t
{
#if defined(_WIN32) || defined(_WIN64)
	HANDLE handle;
#else
	pthread_t handle;
#endif
	int priority;
	void* fn;
	void* arg;
}hpr2_thread_t;

int hpr2_thread_create(hpr2_thread_t** handle, void* fn, void* arg, int stack_size, int priority)
{
	(void)(priority);

	if (!handle || !fn)
	{
		return HPR2_ERR;
	}

	*handle = (hpr2_thread_t*)malloc(sizeof(hpr2_thread_t));
	if (*handle == NULL)
	{
		return HPR2_ERR;
	}

	if (stack_size == 0)
	{
		stack_size = HPR2_THREAD_DEFAULT_STACKSIZE;
	}

#if defined(_WIN32) || defined(_WIN64)
	(*handle)->handle = (HANDLE)_beginthreadex(NULL, stack_size, (unsigned (__stdcall*)(void*))fn, arg, 0, NULL);
	if ((*handle)->handle == NULL)
	{
		free(*handle);
		*handle = NULL;
		return HPR2_ERR;
	}

	(*handle)->arg = arg;
	(*handle)->fn = fn;
	(*handle)->priority = priority;

	return HPR2_OK;
#else
	pthread_attr_t attr;
	int ret = pthread_attr_init(&attr);
	if (ret)
	{
		free(*handle);
		*handle = NULL;
		return HPR2_ERR;
	}

	if (stack_size != 0)
	{
		ret = pthread_attr_setstacksize(&attr, stack_size);
		if (ret != 0)
		{
			free(*handle);
			*handle = NULL;
			pthread_attr_destroy(&attr);
			return HPR2_ERR;
		}
	}

	ret = pthread_create(&(*handle)->handle, &attr, (void *(*)(void*))fn, arg);
	if (ret)
	{
		free(*handle);
		*handle = NULL;
		pthread_attr_destroy(&attr);
		return HPR2_ERR;
	}
	else
	{
		(*handle)->arg = arg;
		(*handle)->fn = fn;
		(*handle)->priority = priority;

		pthread_attr_destroy(&attr);
		return HPR2_OK;
	}
#endif
}

int hpr2_thread_wait(hpr2_thread_t* handle)
{
	if (!handle)
	{
		return HPR2_ERR;
	}

#if defined(_WIN32) || defined(_WIN64)
	if (!handle->handle)
	{
		return HPR2_ERR;
	}

	if (WAIT_OBJECT_0 == WaitForSingleObject(handle->handle, INFINITE))
	{
		CloseHandle(handle->handle);
		handle->handle = NULL;

		free(handle);
		handle = NULL;

		return HPR2_OK;
	}

	return HPR2_ERR;
#else
	int ret = 0;

	ret = pthread_join(handle->handle, NULL); 
	free(handle);
	handle = NULL;

	return (ret == 0) ? HPR2_OK : HPR2_ERR;
#endif
}

int hpr2_thread_detached_create(void*fn, void* arg, int stack_size, int priority)
{
	(void)(priority);

	if (!fn)
	{
		return HPR2_ERR;
	}

	if (stack_size == 0)
	{
		stack_size = HPR2_THREAD_DEFAULT_STACKSIZE;
	}

#if defined(_WIN32) || defined(_WIN64)
	HANDLE handle = (HANDLE)_beginthreadex(NULL, stack_size, (unsigned (__stdcall*)(void*))fn, arg, 0, NULL);
	if (handle == NULL)
	{
		return HPR2_ERR;
	}

	CloseHandle(handle);

	return HPR2_OK;
#else
	pthread_attr_t attr;
	int ret = pthread_attr_init(&attr);
	if (ret)
	{
		return HPR2_ERR;
	}

	ret = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	if (ret)
	{
		pthread_attr_destroy(&attr);
		return HPR2_ERR;
	}

	if (stack_size != 0)
	{
		ret = pthread_attr_setstacksize(&attr, stack_size);
		if (ret != 0)
		{
			pthread_attr_destroy(&attr);
			return HPR2_ERR;
		}
	}

	pthread_t id;
	ret = pthread_create(&id, &attr, (void *(*)(void*))fn, arg);
	if (ret)
	{
		pthread_attr_destroy(&attr);
		return HPR2_ERR;
	}
	else
	{
		pthread_attr_destroy(&attr);
		return HPR2_OK;
	}
#endif
}

hpr2_uint64 hpr2_thread_get_selfid()
{
#if defined(_WIN32) || defined(_WIN64)
	return (hpr2_uint64)GetCurrentThreadId();
#else
	return (hpr2_uint64)pthread_self();
#endif
}

